"""
# Plotting Benchmark Results

## Usage

    usage: benchmark_plot.py [-h] [--test TEST] [--kind KIND] [--show] [--save]
                             filename name

    Plotting

    positional arguments:
      filename     path to file with benchmark data
      name         type name

    optional arguments:
      -h, --help   show this help message and exit
      --test TEST
      --kind KIND
      --show
      --save

## Compare Implementations

    python -m tests.benchmark_sortedlist --bare > tests/results_sortedlist.txt
    python -m tests.benchmark_plot tests/results_sortedlist.txt SortedList --save

    python -m tests.benchmark_sorteddict --bare > tests/results_sorteddict.txt
    python -m tests.benchmark_plot tests/results_sorteddict.txt SortedDict --save

    python -m tests.benchmark_sortedset --bare > tests/results_sortedset.txt
    python -m tests.benchmark_plot tests/results_sortedset.txt SortedSet --save

## Compare Python Versions

    rm tests/results_runtime_sortedlist.txt
    echo ". env27/bin/activate && python -m tests.benchmark_sortedlist --bare --kind SortedList --suffix _Py27 >> tests/results_runtime_sortedlist.txt" | bash
    echo ". env34/bin/activate && python -m tests.benchmark_sortedlist --bare --kind SortedList --suffix _Py34 >> tests/results_runtime_sortedlist.txt" | bash
    echo ". env27/bin/activate && pypy -m tests.benchmark_sortedlist --bare --kind SortedList --suffix _PyPy >> tests/results_runtime_sortedlist.txt" | bash
    python -m tests.benchmark_plot tests/results_runtime_sortedlist.txt SortedList --suffix _runtime --save

    rm tests/results_runtime_sorteddict.txt
    echo ". env27/bin/activate && python -m tests.benchmark_sorteddict --bare --kind SortedDict --suffix _Py27 >> tests/results_runtime_sorteddict.txt" | bash
    echo ". env34/bin/activate && python -m tests.benchmark_sorteddict --bare --kind SortedDict --suffix _Py34 >> tests/results_runtime_sorteddict.txt" | bash
    echo ". env27/bin/activate && pypy -m tests.benchmark_sorteddict --bare --kind SortedDict --suffix _PyPy >> tests/results_runtime_sorteddict.txt" | bash
    python -m tests.benchmark_plot tests/results_runtime_sorteddict.txt SortedDict --suffix _runtime --save

    rm tests/results_runtime_sortedset.txt
    echo ". env27/bin/activate && python -m tests.benchmark_sortedset --bare --kind SortedSet --suffix _Py27 >> tests/results_runtime_sortedset.txt" | bash
    echo ". env34/bin/activate && python -m tests.benchmark_sortedset --bare --kind SortedSet --suffix _Py34 >> tests/results_runtime_sortedset.txt" | bash
    echo ". env27/bin/activate && pypy -m tests.benchmark_sortedset --bare --kind SortedSet --suffix _PyPy >> tests/results_runtime_sortedset.txt" | bash
    python -m tests.benchmark_plot tests/results_runtime_sortedset.txt SortedSet --suffix _runtime --save

## Compare Loads

    rm tests/results_load_sortedlist.txt
    python -m tests.benchmark_sortedlist --bare --kind SortedList --suffix _100 --load 100 --no-limit >> tests/results_load_sortedlist.txt
    python -m tests.benchmark_sortedlist --bare --kind SortedList --suffix _1000 --load 1000 --no-limit >> tests/results_load_sortedlist.txt
    python -m tests.benchmark_sortedlist --bare --kind SortedList --suffix _10000 --load 10000 --no-limit >> tests/results_load_sortedlist.txt
    python -m tests.benchmark_plot tests/results_load_sortedlist.txt SortedList --suffix _load --save

    rm tests/results_load_sorteddict.txt
    python -m tests.benchmark_sorteddict --bare --kind SortedDict --suffix _100 --load 100 --no-limit >> tests/results_load_sorteddict.txt
    python -m tests.benchmark_sorteddict --bare --kind SortedDict --suffix _1000 --load 1000 --no-limit >> tests/results_load_sorteddict.txt
    python -m tests.benchmark_sorteddict --bare --kind SortedDict --suffix _10000 --load 10000 --no-limit >> tests/results_load_sorteddict.txt
    python -m tests.benchmark_plot tests/results_load_sorteddict.txt SortedDict --suffix _load --save

    rm tests/results_load_sortedset.txt
    python -m tests.benchmark_sortedset --bare --kind SortedSet --suffix _100 --load 100 --no-limit >> tests/results_load_sortedset.txt
    python -m tests.benchmark_sortedset --bare --kind SortedSet --suffix _1000 --load 1000 --no-limit >> tests/results_load_sortedset.txt
    python -m tests.benchmark_sortedset --bare --kind SortedSet --suffix _10000 --load 10000 --no-limit >> tests/results_load_sortedset.txt
    python -m tests.benchmark_plot tests/results_load_sortedset.txt SortedSet --suffix _load --save
"""

from __future__ import print_function

import argparse
import matplotlib.pyplot as plt
from collections import defaultdict

def tree():
    return defaultdict(tree)

parser = argparse.ArgumentParser(description='Plotting')
parser.add_argument('filename', help='path to file with benchmark data')
parser.add_argument('name', help='type name')
parser.add_argument('--test', action='append', help='filter tests by name')
parser.add_argument('--kind', action='append', help='filter types by name')
parser.add_argument('--suffix', default='', help='suffix for output')
parser.add_argument('--show', action='store_true', help='show results')
parser.add_argument('--save', action='store_true', help='save results')

args = parser.parse_args()

text = open(args.filename).read()

lines = text.splitlines()
lines = [line.split() for line in lines]
for line in lines:
    line[2] = int(line[2])
    line[3:] = map(float, line[3:])
data = tree()
for line in lines:
    data[line[0]][line[1]][line[2]] = line

def order_kinds(kinds):
    for idx, kind in enumerate(kinds):
        if kind.startswith('Sorted'):
            del kinds[idx]
            kinds.insert(0, kind)
            break
    return kinds

def test_plot(test):
    ax = plt.gca()
    ax.set_color_cycle(['b', 'g', 'r', 'c', 'm', 'y', 'k', '0.8'])
    kinds = order_kinds(sorted(args.kind or list(data[test])))
    for kind in kinds:
        kind_plot(test, kind)
    plt.ylim(ymin=9e-7)
    plt.loglog()
    plt.title(args.name + ' Performance: ' + test)
    plt.ylabel('Seconds')
    plt.xlabel('List Size')
    plt.legend(kinds, loc=2)

def kind_plot(test, kind):
    sizes = sorted(data[test][kind].keys())
    yrange = [[(data[test][kind][size][5] - data[test][kind][size][3])
               for size in sizes],
              [(data[test][kind][size][4] - data[test][kind][size][5])
               for size in sizes]]
    # Timer isn't any better than micro-second resolution.
    yvalues = [max(1e-6, data[test][kind][size][5]) for size in sizes]
    plt.errorbar(sizes, yvalues, yerr=yrange)

tests = args.test or list(data)

for test in tests:
    test_plot(test)

    if args.show:
        plt.show()

    if args.save:
        plt.savefig('{0}{1}-{2}.png'.format(args.name, args.suffix, test))

    plt.close()



"""
Benchmark Sorted List Datatypes
"""

import warnings
from .benchmark import *

# Tests.

@register_test
def add(func, size):
    for val in lists[size][::100]:
        func(val)

@register_test
def update_small(func, size):
    func(lists[size][::10])

@register_test
def update_large(func, size):
    func(lists[size])

@register_test
def contains(func, size):
    for val in lists[size][::100]:
        assert func(val)

@register_test
def remove(func, size):
    for val in lists[size][::100]:
        func(val)

@register_test
def delitem(func, size):
    for val in range(int(size / 100)):
        pos = random.randrange(size - val)
        func(pos)

@register_test
def bisect(func, size):
    for val in lists[size][::100]:
        func(val)

@register_test
def getitem(func, size):
    for val in lists[size][::100]:
        assert func(val) == val

@register_test
def pop(func, size):
    for val in range(int(size / 100)):
        assert func() == (size - val - 1)

@register_test
def index(func, size):
    for val in lists[size][::100]:
        assert func(val) == val

@register_test
def iter(func, size):
    assert all(idx == val for idx, val in enumerate(func()))

@register_test
def count(func, size):
    for val in lists[size][::100]:
        assert func(val) == 1

@register_test
def priorityqueue(func, size):
    for val in lists[size][::10]:
        func(val)

@register_test
def multiset(func, size):
    for val in lists[size][::10]:
        func(val)

@register_test
def ranking(func, size):
    for val in lists[size][::10]:
        func(val)

@register_test
def neighbor(func, size):
    for val in lists[size][::10]:
        func(val)

@register_test
def intervals(func, size):
    for val in lists[size][::10]:
        func(val)

@register_test
def init(func, size):
    func(lists[size])

# Setups.

def do_nothing(obj, size):
    pass

def fill_values(obj, size):
    obj.update(sorted(lists[size]))

# Implementation imports.

from .context import sortedcontainers
from sortedcontainers import SortedList
kinds['SortedList'] = SortedList

from sortedcontainers import SortedListWithKey
kinds['SortedListWithKey'] = SortedListWithKey

try:
    from blist import sortedlist
    kinds['blist.sortedlist'] = sortedlist
    from functools import partial
    def identity(value):
        return value
    kinds['blist.sortedlist(key=identity)'] = partial(sortedlist, key=identity)
except ImportError:
    warnings.warn('No module named blist', ImportWarning)

try:
    from sortedcollection import SortedCollection
    from bisect import bisect_left

    SortedCollection.add = SortedCollection.insert

    def update(self, iterable):
        for value in iterable:
            self.insert(value)
    SortedCollection.update = update

    def bisect(self, item):
        key = self._key(item)
        pos = bisect_left(self._keys, key)
        return pos
    SortedCollection.bisect = bisect

    def pop(self):
        self._keys.pop()
        return self._items.pop()
    SortedCollection.pop = pop

    def discard(self, item):
        try:
            self.remove(item)
        except ValueError:
            pass
    SortedCollection.discard = discard

    kinds['sortedcollection'] = SortedCollection
except ImportError:
    warnings.warn('No module named sortedcollection', ImportWarning)

# Implementation configuration.

for name in tests:
    impls[name] = OrderedDict()

for name, kind in kinds.items():
    impls['add'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'add',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['update_small'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'update',
        'limit': 1000000
    }
limit('update_small', 'sortedcollection', 100000)

for name, kind in kinds.items():
    impls['update_large'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'update',
        'limit': 1000000
    }
limit('update_large', 'sortedcollection', 100000)

for name, kind in kinds.items():
    impls['contains'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': '__contains__',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['remove'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'remove',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['delitem'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': '__delitem__',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['bisect'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'bisect',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['getitem'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': '__getitem__',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['pop'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'pop',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['index'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'index',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['iter'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': '__iter__',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['count'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'count',
        'limit': 1000000
    }

class Mixed:
    def __init__(self, kind):
        self.rand = random.Random(0)
        self.kind = kind
    def __call__(self, *args, **kwargs):
        self.obj = self.kind(*args, **kwargs)
        return self
    def update(self, values):
        self.obj.update(values)
    def run(self, value):
        raise NotImplementedError

class PriorityQueue(Mixed):
    def run(self, value):
        """
        40% add
        40% pop
        10% discard
        9% contains
        1% iter (first 100 elements)
        """
        obj = self.obj
        pos = self.rand.randrange(100)

        if pos < 40:
            obj.add(value)
        elif pos < 80:
            obj.pop()
        elif pos < 90:
            obj.discard(value)
        elif pos < 99:
            value in obj
        else:
            for idx, temp in enumerate(obj):
                if idx > 100:
                    break

for name, kind in kinds.items():
    impls['priorityqueue'][name] = {
        'setup': fill_values,
        'ctor': PriorityQueue(kind),
        'func': 'run',
        'limit': 1000000
    }
limit('priorityqueue', 'sortedcollection', 100000)

class Multiset(Mixed):
    def run(self, value):
        """
        75% contains
        10% add
        10% remove
        5% getitem
        """
        obj = self.obj
        pos = self.rand.randrange(100)

        if pos < 75:
            assert value in obj
        elif pos < 85:
            obj.add(value)
        elif pos < 95:
            obj.remove(value)
        else:
            if value < len(obj):
                obj[value]

for name, kind in kinds.items():
    impls['multiset'][name] = {
        'setup': fill_values,
        'ctor': Multiset(kind),
        'func': 'run',
        'limit': 1000000
    }
limit('multiset', 'sortedcollection', 100000)

class Ranking(Mixed):
    def run(self, value):
        """
        40% getitem
        40% index
        10% add
        10% remove
        """
        obj = self.obj
        pos = self.rand.randrange(100)

        if pos < 40:
            if value < len(obj):
                obj[value]
        elif pos < 80:
            assert obj.index(value) >= 0
        elif pos < 90:
            obj.add(value)
        else:
            obj.remove(value)

for name, kind in kinds.items():
    impls['ranking'][name] = {
        'setup': fill_values,
        'ctor': Ranking(kind),
        'func': 'run',
        'limit': 1000000
    }
limit('ranking', 'sortedcollection', 100000)

class Neighbor(Mixed):
    def run(self, value):
        """
        75% bisect
        10% add
        10% remove
        5% iter (first 100 elements)
        """
        obj = self.obj
        pos = self.rand.randrange(100)

        if pos < 75:
            obj.bisect(value)
        elif pos < 85:
            obj.add(value)
        elif pos < 95:
            obj.remove(value)
        else:
            for idx, temp in enumerate(obj):
                if idx > 100:
                    break

for name, kind in kinds.items():
    impls['neighbor'][name] = {
        'setup': fill_values,
        'ctor': Neighbor(kind),
        'func': 'run',
        'limit': 1000000
    }
limit('neighbor', 'sortedcollection', 100000)

class Intervals(Mixed):
    def run(self, value):
        """
        30% bisect
        20% getitem
        20% delitem
        10% get-slice (range query)
        10% add
        10% discard
        """
        obj = self.obj
        pos = self.rand.randrange(100)

        if pos < 30:
            obj.bisect(value)
        elif pos < 50:
            if value < len(obj):
                obj[value]
        elif pos < 70:
            if value < len(obj):
                del obj[value]
        elif pos < 80:
            if value < len(obj):
                limit = min(value + 100, len(obj))
                other = self.rand.randrange(value, limit)
                obj[value:other]
        elif pos < 90:
            obj.add(value)
        else:
            obj.discard(value)

for name, kind in kinds.items():
    impls['intervals'][name] = {
        'setup': fill_values,
        'ctor': Intervals(kind),
        'func': 'run',
        'limit': 1000000
    }
limit('intervals', 'sortedcollection', 100000)

for name, kind in kinds.items():
    impls['init'][name] = {
        'setup': do_nothing,
        'ctor': kind,
        'func': '__init__',
        'limit': 1000000
    }
limit('init', 'blist.sortedlist', 100000)
limit('init', 'blist.sortedlist(key=identity)', 100000)

if __name__ == '__main__':
    main('SortedList')



"""
Benchmark Sorted Dictionary Datatypes
"""

import warnings
from .benchmark import *

# Tests.

@register_test
def contains(func, size):
    for val in lists[size][::100]:
        assert func(val)

@register_test
def getitem(func, size):
    for val in lists[size][::100]:
        assert func(val) == -val

@register_test
def setitem(func, size):
    for val in lists[size][::100]:
        func(val, -val)

@register_test
def setitem_existing(func, size):
    for val in lists[size][::100]:
        func(val, -val)

@register_test
def delitem(func, size):
    for val in lists[size][::100]:
        func(val)

@register_test
def iter(func, size):
    assert all(idx == val for idx, val in enumerate(func()))

@register_test
def init(func, size):
    func((val, -val) for val in lists[size])

# Setups.

def do_nothing(obj, size):
    pass

def fill_values(obj, size):
    if hasattr(obj, 'update'):
        obj.update({val: -val for val in range(size)})
    else:
        for val in range(size):
            obj[val] = -val

# Implementation imports.

from .context import sortedcontainers
from sortedcontainers import SortedDict
kinds['SortedDict'] = SortedDict

try:
    from rbtree import rbtree
    kinds['rbtree'] = rbtree
except ImportError:
    warnings.warn('No module named rbtree', ImportWarning)

try:
    from blist import sorteddict
    kinds['blist.sorteddict'] = sorteddict
except ImportError:
    warnings.warn('No module named blist', ImportWarning)

try:
    from treap import treap
    kinds['treap'] = treap
except ImportError:
    warnings.warn('No module named treap', ImportWarning)

try:
    from bintrees import FastAVLTree, FastRBTree
    kinds['FastAVLTree'] = FastAVLTree
    kinds['FastRBTree'] = FastRBTree
except ImportError:
    warnings.warn('No module named bintrees', ImportWarning)

try:
    from skiplistcollections import SkipListDict
    kinds['SkipListDict'] = SkipListDict
except ImportError:
    warnings.warn('No module named skiplistcollections', ImportWarning)

try:
    from banyan import SortedDict as BanyanSortedDict
    kinds['banyan.SortedDict'] = BanyanSortedDict
except ImportError:
    warnings.warn('No module named banyan', ImportWarning)

# Implementation configuration.

for name in tests:
    impls[name] = OrderedDict()

for name, kind in kinds.items():
    impls['contains'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': '__contains__',
        'limit': 1000000
    }

if 'treap' in impls['contains']:
    del impls['contains']['treap']

for name, kind in kinds.items():
    impls['getitem'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': '__getitem__',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['setitem'][name] = {
        'setup': do_nothing,
        'ctor': kind,
        'func': '__setitem__',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['setitem_existing'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': '__setitem__',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['delitem'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': '__delitem__',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['iter'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': '__iter__',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['init'][name] = {
        'setup': do_nothing,
        'ctor': kind,
        'func': 'update',
        'limit': 1000000
    }

remove('init', 'rbtree')
remove('init', 'treap')

limit('init', 'blist.sorteddict', 100000)
limit('init', 'SkipListDict', 100000)

if __name__ == '__main__':
    main('SortedDict')



from __future__ import print_function
from sys import hexversion

import time, random, argparse
from functools import partial
try:
    from collections import OrderedDict
except:
    from ordereddict import OrderedDict

if hexversion < 0x03000000:
    range = xrange
    name_attr = 'func_name'
else:
    name_attr = '__name__'

def detail(*values, **kwargs):
    if not args.bare:
        print(*values, **kwargs)

def measure(test, func, size):
    start = time.clock()
    test(func, size)
    end = time.clock()
    return (end - start)

def benchmark(test, name, ctor, setup, func_name, limit):
    if args.load > 0:
        if name == 'SortedDict':
            ctor = partial(ctor, args.load)
        else:
            ctor = partial(ctor, load=args.load)

    for size in sizes:
        if not args.no_limit and size > limit:
            continue

        # warmup

        obj = ctor()
        setup(obj, size)
        func = getattr(obj, func_name)
        measure(test, func, size)
        
        # record

        times = []
        for rpt in range(5):
            obj = ctor()
            setup(obj, size)
            func = getattr(obj, func_name)
            times.append(measure(test, func, size))

        print(getattr(test, name_attr), name + args.suffix, size, min(times),
              max(times), times[2], sum(times) / len(times))

def register_test(func):
    tests[getattr(func, name_attr)] = func
    return func

def limit(test, kind, value):
    if kind in impls[test]:
        impls[test][kind]['limit'] = value

def remove(test, kind):
    if kind in impls[test]:
        del impls[test][kind]

tests = OrderedDict()
kinds = OrderedDict()
impls = OrderedDict()
sizes = []
lists = {}

parser = argparse.ArgumentParser(description='Benchmarking')
parser.add_argument('--seed', type=int, default=0,
                    help='seed value for random')
parser.add_argument('--no-limit', default=False, action='store_true',
                    help='no limit on size')
parser.add_argument('--test', action='append', help='filter tests by name')
parser.add_argument('--kind', action='append', help='filter types by name')
parser.add_argument('--size', type=int, action='append',
                    help='specify sizes to test')
parser.add_argument('--suffix', default='', help='suffix for kind name')
parser.add_argument('--bare', action='store_true', default=False,
                    help='hide header and footer info')
parser.add_argument('--load', type=int, default=0,
                    help='load value for sorted container types')
args = parser.parse_args()

def main(name):
    global sizes, lists

    detail('Benchmarking:', name)

    detail('Seed:', args.seed)
    random.seed(args.seed)

    sizes.extend(args.size or [100, 1000, 10000, 100000, 1000000, 10000000])

    detail('Sizes:', sizes)

    lists.update((key, list(range(key))) for key in sizes)
    for key in sizes:
        random.shuffle(lists[key])

    test_names = args.test or tests.keys()
    kind_names = args.kind or kinds.keys()

    detail('Tests:', list(test_names))
    detail('Kinds:', list(kind_names))

    detail('test_name', 'data_type', 'size', 'min', 'max', 'median', 'mean')

    for test in impls:
        if test not in test_names:
            continue
        for name in impls[test]:
            if name not in kind_names:
                continue
            details = impls[test][name]
            benchmark(tests[test], name, details['ctor'], details['setup'], details['func'], details['limit'])

    detail('Benchmark Stop')



# -*- coding: utf-8 -*-

from sys import hexversion

import random
from .context import sortedcontainers
from sortedcontainers import SortedList, SortedListWithKey
from nose.tools import raises

if hexversion < 0x03000000:
    from itertools import izip as zip
    range = xrange

def modulo(val):
    return val % 10

def test_init():
    slt = SortedListWithKey(key=modulo)
    slt._check()

    slt = SortedListWithKey(load=10000, key=modulo)
    assert slt._load == 10000
    assert slt._twice == 20000
    assert slt._half == 5000
    slt._check()

    slt = SortedListWithKey(range(10000), key=modulo)
    assert all(tup[0] == tup[1] for tup in zip(slt, sorted(range(10000), key=modulo)))

    slt.clear()
    assert slt._len == 0
    assert slt._maxes == []
    assert slt._lists == []

    assert isinstance(slt, SortedList)
    assert isinstance(slt, SortedListWithKey)

    slt._check()

def test_new():
    slt = SortedList(key=modulo)
    slt._check()

    assert isinstance(slt, SortedList)
    assert isinstance(slt, SortedListWithKey)
    assert type(slt) == SortedListWithKey

    slt = SortedListWithKey(key=modulo)
    slt._check()

    assert isinstance(slt, SortedList)
    assert isinstance(slt, SortedListWithKey)
    assert type(slt) == SortedListWithKey

@raises(TypeError)
def test_new_error():
    class SortedListPlus(SortedList):
        pass
    SortedListPlus(key=modulo)

def test_key():
    slt = SortedListWithKey(range(10000), key=lambda val: val % 10)
    slt._check()

    values = sorted(range(10000), key=lambda val: (val % 10, val))
    assert slt == values
    assert all(val in slt for val in range(10000))

def test_key2():
    class Incomparable:
        pass
    a = Incomparable()
    b = Incomparable()
    slt = SortedListWithKey(key=lambda val: 1)
    slt.add(a)
    slt.add(b)
    assert slt == [a, b]

def test_add():
    random.seed(0)
    slt = SortedListWithKey(key=modulo)
    for val in range(1000):
        slt.add(val)
    slt._check()

    slt = SortedListWithKey(key=modulo)
    for val in range(1000, 0, -1):
        slt.add(val)
    slt._check()

    slt = SortedListWithKey(key=modulo)
    for val in range(1000):
        slt.add(random.random())
    slt._check()

def test_update():
    slt = SortedListWithKey(key=modulo)

    slt.update(range(1000))
    assert all(tup[0] == tup[1] for tup in zip(slt, sorted(range(1000), key=modulo)))
    assert len(slt) == 1000
    slt._check()

    slt.update(range(10000))
    assert len(slt) == 11000
    slt._check()

def test_contains():
    slt = SortedListWithKey(key=modulo, load=7)

    assert 0 not in slt

    slt.update(range(100))

    for val in range(100):
        assert val in slt

    assert 100 not in slt

    slt._check()

    slt = SortedListWithKey(range(100), key=modulo, load=4)
    assert all(val not in slt for val in range(100, 200))

def test_discard():
    slt = SortedListWithKey(key=modulo)

    assert slt.discard(0) == None
    assert len(slt) == 0
    slt._check()

    slt = SortedListWithKey([1, 2, 2, 2, 3, 3, 5], load=4, key=modulo)

    slt.discard(6)
    slt._check()
    slt.discard(4)
    slt._check()
    slt.discard(2)
    slt._check()
    slt.discard(11)
    slt.discard(12)
    slt.discard(13)
    slt.discard(15)

    assert all(tup[0] == tup[1] for tup in zip(slt, [1, 2, 2, 3, 3, 5]))

def test_remove():
    slt = SortedListWithKey(key=modulo)

    assert slt.discard(0) == None
    assert len(slt) == 0
    slt._check()

    slt = SortedListWithKey([1, 2, 2, 2, 3, 3, 5], load=4, key=modulo)

    slt.remove(2)
    slt._check()

    assert all(tup[0] == tup[1] for tup in zip(slt, [1, 2, 2, 3, 3, 5]))

@raises(ValueError)
def test_remove_valueerror1():
    slt = SortedListWithKey(key=modulo)
    slt.remove(0)

@raises(ValueError)
def test_remove_valueerror2():
    slt = SortedListWithKey(range(100), load=10, key=modulo)
    slt.remove(100)

@raises(ValueError)
def test_remove_valueerror3():
    slt = SortedListWithKey([1, 2, 2, 2, 3, 3, 5], key=modulo)
    slt.remove(4)

@raises(ValueError)
def test_remove_valueerror4():
    slt = SortedListWithKey([1, 1, 1, 2, 2, 2], key=modulo)
    slt.remove(13)

@raises(ValueError)
def test_remove_valueerror5():
    slt = SortedListWithKey([1, 1, 1, 2, 2, 2], key=modulo)
    slt.remove(12)

def test_delete():
    slt = SortedListWithKey(range(20), load=4, key=modulo)
    slt._check()
    for val in range(20):
        slt.remove(val)
        slt._check()
    assert len(slt) == 0
    assert slt._maxes == []
    assert slt._lists == []

def test_getitem():
    random.seed(0)
    slt = SortedListWithKey(load=17, key=modulo)

    slt.append(5)
    slt._build_index()
    slt._check()
    slt.clear()

    lst = list(random.random() for rpt in range(100))
    slt.update(lst)
    lst.sort(key=modulo)

    assert all(slt[idx] == lst[idx] for idx in range(100))
    assert all(slt[idx - 99] == lst[idx - 99] for idx in range(100))

def test_getitem_slice():
    random.seed(0)
    slt = SortedListWithKey(load=17, key=modulo)

    lst = list()

    for rpt in range(100):
        val = random.random()
        slt.add(val)
        lst.append(val)

    lst.sort(key=modulo)

    assert all(slt[start:] == lst[start:]
               for start in [-75, -25, 0, 25, 75])

    assert all(slt[:stop] == lst[:stop]
               for stop in [-75, -25, 0, 25, 75])

    assert all(slt[::step] == lst[::step]
               for step in [-5, -1, 1, 5])

    assert all(slt[start:stop] == lst[start:stop]
               for start in [-75, -25, 0, 25, 75]
               for stop in [-75, -25, 0, 25, 75])

    assert all(slt[:stop:step] == lst[:stop:step]
               for stop in [-75, -25, 0, 25, 75]
               for step in [-5, -1, 1, 5])

    assert all(slt[start::step] == lst[start::step]
               for start in [-75, -25, 0, 25, 75]
               for step in [-5, -1, 1, 5])

    assert all(slt[start:stop:step] == lst[start:stop:step]
               for start in [-75, -25, 0, 25, 75]
               for stop in [-75, -25, 0, 25, 75]
               for step in [-5, -1, 1, 5])

def test_getitem_slice_big():
    slt = SortedListWithKey(range(4), key=modulo)
    lst = sorted(range(4), key=modulo)

    itr = ((start, stop, step)
           for start in [-6, -4, -2, 0, 2, 4, 6]
           for stop in [-6, -4, -2, 0, 2, 4, 6]
           for step in [-3, -2, -1, 1, 2, 3])

    for start, stop, step in itr:
        assert slt[start:stop:step] == lst[start:stop:step]

@raises(ValueError)
def test_getitem_slicezero():
    slt = SortedListWithKey(range(100), load=17, key=modulo)
    slt[::0]

@raises(IndexError)
def test_getitem_indexerror1():
    slt = SortedListWithKey(key=modulo)
    slt[5]

@raises(IndexError)
def test_getitem_indexerror2():
    slt = SortedListWithKey(range(100), key=modulo)
    slt[200]

@raises(IndexError)
def test_getitem_indexerror3():
    slt = SortedListWithKey(range(100), key=modulo)
    slt[-101]

def test_delitem():
    random.seed(0)

    slt = SortedListWithKey(range(100), load=17, key=modulo)
    while len(slt) > 0:
        del slt[random.randrange(len(slt))]
        slt._check()

    slt = SortedListWithKey(range(100), load=17, key=modulo)
    del slt[:]
    assert len(slt) == 0
    slt._check()

def test_delitem_slice():
    slt = SortedListWithKey(range(100), load=17, key=modulo)
    del slt[10:40:1]
    del slt[10:40:-1]
    del slt[10:40:2]
    del slt[10:40:-2]

def test_setitem():
    random.seed(0)
    slt = SortedListWithKey(range(0, 100), load=17, key=modulo)
    slt[0] = 100
    slt[99] = 99
    slt[55] = 45

def test_setitem_slice():
    slt = SortedListWithKey(range(100), load=17, key=modulo)
    slt[:10] = [90, 80, 70, 60, 50, 40, 30, 20, 10, 0]
    slt[:10:2] = [0, 10, 20, 30, 40]
    slt[:] = sorted(range(100), key=modulo)
    slt[90:] = []
    slt[:10] = []
    assert len(slt) == 80

@raises(ValueError)
def test_setitem_slice_bad():
    slt = SortedListWithKey(range(100), load=17, key=modulo)
    slt[:10] = list(reversed(range(10)))

@raises(ValueError)
def test_setitem_slice_bad1():
    slt = SortedListWithKey(range(100), load=17, key=modulo)
    slt[10:20] = range(20, 30)

@raises(ValueError)
def test_setitem_slice_bad2():
    slt = SortedListWithKey(range(100), load=17, key=modulo)
    slt[20:30] = range(10, 20)

@raises(ValueError)
def test_setitem_extended_slice_bad1():
    slt = SortedListWithKey(range(100), load=17, key=modulo)
    slt[20:80:3] = list(range(10))

@raises(ValueError)
def test_setitem_extended_slice_bad2():
    slt = SortedListWithKey(range(100), load=17, key=modulo)
    slt[40:90:5] = list(range(10))

@raises(ValueError)
def test_setitem_valueerror1():
    slt = SortedListWithKey(range(10), key=modulo)
    slt[9] = 10

@raises(ValueError)
def test_setitem_valueerror2():
    slt = SortedListWithKey(range(10), key=modulo)
    slt[0] = 9

def test_iter():
    slt = SortedListWithKey(range(10000), key=modulo)
    itr = iter(slt)
    assert all(tup[0] == tup[1] for tup in zip(sorted(range(10000), key=modulo), itr))

def test_reversed():
    slt = SortedListWithKey(range(10000), key=modulo)
    rev = reversed(slt)
    assert all(tup[0] == tup[1] for tup in zip(reversed(sorted(range(10000), key=modulo)), rev))

def test_islice():
    sl = SortedListWithKey(load=7, key=modulo)

    assert [] == list(sl.islice())

    values = sorted(range(100), key=modulo)
    sl.update(values)

    for start in range(53):
        for stop in range(53):
            assert list(sl.islice(start, stop)) == values[start:stop]

    for start in range(53):
        for stop in range(53):
            assert list(sl.islice(start, stop, reverse=True)) == values[start:stop][::-1]

    for start in range(53):
        assert list(sl.islice(start=start)) == values[start:]
        assert list(sl.islice(start=start, reverse=True)) == values[start:][::-1]

    for stop in range(53):
        assert list(sl.islice(stop=stop)) == values[:stop]
        assert list(sl.islice(stop=stop, reverse=True)) == values[:stop][::-1]

def test_irange():
    values = sorted(range(100), key=modulo)

    for load in range(5, 16):
        slt = SortedListWithKey(range(100), load=load, key=modulo)

        for start in range(10):
            for end in range(start, 10):
                temp = list(slt.irange(start, end))
                assert temp == values[(start * 10):((end + 1) * 10)]

                temp = list(slt.irange(start, end, reverse=True))
                assert temp == values[(start * 10):((end + 1) * 10)][::-1]

        for start in range(10):
            for end in range(start, 10):
                temp = list(slt.irange(start, end, inclusive=(True, False)))
                assert temp == values[(start * 10):(end * 10)]

        for start in range(10):
            for end in range(start, 10):
                temp = list(slt.irange(start, end, (False, True)))
                assert temp == values[((start + 1) * 10):((end + 1) * 10)]

        for start in range(10):
            for end in range(start, 10):
                temp = list(slt.irange(start, end, inclusive=(False, False)))
                assert temp == values[((start + 1) * 10):(end * 10)]

        for start in range(10):
            temp = list(slt.irange(minimum=start))
            assert temp == values[(start * 10):]

        for end in range(10):
            temp = list(slt.irange(maximum=end))
            assert temp == values[:(end + 1) * 10]

def test_irange_key():
    values = sorted(range(100), key=modulo)

    for load in range(5, 16):
        slt = SortedListWithKey(range(100), load=load, key=modulo)

        for start in range(10):
            for end in range(start, 10):
                temp = list(slt.irange_key(start, end))
                assert temp == values[(start * 10):((end + 1) * 10)]

                temp = list(slt.irange_key(start, end, reverse=True))
                assert temp == values[(start * 10):((end + 1) * 10)][::-1]

        for start in range(10):
            for end in range(start, 10):
                temp = list(slt.irange_key(start, end, inclusive=(True, False)))
                assert temp == values[(start * 10):(end * 10)]

        for start in range(10):
            for end in range(start, 10):
                temp = list(slt.irange_key(start, end, (False, True)))
                assert temp == values[((start + 1) * 10):((end + 1) * 10)]

        for start in range(10):
            for end in range(start, 10):
                temp = list(slt.irange_key(start, end, inclusive=(False, False)))
                assert temp == values[((start + 1) * 10):(end * 10)]

        for start in range(10):
            temp = list(slt.irange_key(min_key=start))
            assert temp == values[(start * 10):]

        for end in range(10):
            temp = list(slt.irange_key(max_key=end))
            assert temp == values[:(end + 1) * 10]

def test_len():
    slt = SortedListWithKey(key=modulo)

    for val in range(10000):
        slt.add(val)
        assert len(slt) == (val + 1)

def test_bisect_left():
    slt = SortedListWithKey(key=modulo)
    assert slt.bisect_left(0) == 0
    slt = SortedListWithKey(range(100), load=17, key=modulo)
    slt.update(range(100))
    slt._check()
    assert slt.bisect_left(50) == 0
    assert slt.bisect_left(0) == 0

def test_bisect():
    slt = SortedListWithKey(key=modulo)
    assert slt.bisect(10) == 0
    slt = SortedListWithKey(range(100), load=17, key=modulo)
    slt.update(range(100))
    slt._check()
    assert slt.bisect(10) == 20
    assert slt.bisect(0) == 20

def test_bisect_right():
    slt = SortedListWithKey(key=modulo)
    assert slt.bisect_right(10) == 0
    slt = SortedListWithKey(range(100), load=17, key=modulo)
    slt.update(range(100))
    slt._check()
    assert slt.bisect_right(10) == 20
    assert slt.bisect_right(0) == 20

def test_bisect_key_left():
    slt = SortedListWithKey(key=modulo)
    assert slt.bisect_key_left(10) == 0
    slt = SortedListWithKey(range(100), load=17, key=modulo)
    slt.update(range(100))
    slt._check()
    assert slt.bisect_key_left(0) == 0
    assert slt.bisect_key_left(5) == 100
    assert slt.bisect_key_left(10) == 200

def test_bisect_key_right():
    slt = SortedListWithKey(key=modulo)
    assert slt.bisect_key_right(0) == 0
    slt = SortedListWithKey(range(100), load=17, key=modulo)
    slt.update(range(100))
    slt._check()
    assert slt.bisect_key_right(0) == 20
    assert slt.bisect_key_right(5) == 120
    assert slt.bisect_key_right(10) == 200

def test_bisect_key():
    slt = SortedListWithKey(key=modulo)
    assert slt.bisect_key(0) == 0
    slt = SortedListWithKey(range(100), load=17, key=modulo)
    slt.update(range(100))
    slt._check()
    assert slt.bisect_key(0) == 20
    assert slt.bisect_key(5) == 120
    assert slt.bisect_key(10) == 200

def test_copy():
    slt = SortedListWithKey(range(100), load=7, key=modulo)
    two = slt.copy()
    slt.add(100)
    assert len(slt) == 101
    assert len(two) == 100

def test_copy_copy():
    import copy
    slt = SortedListWithKey(range(100), load=7, key=modulo)
    two = copy.copy(slt)
    slt.add(100)
    assert len(slt) == 101
    assert len(two) == 100

def test_count():
    slt = SortedListWithKey(load=7, key=modulo)

    assert slt.count(0) == 0

    for iii in range(100):
        for jjj in range(iii):
            slt.add(iii)
    slt._check()

    for iii in range(100):
        assert slt.count(iii) == iii

    slt = SortedListWithKey(range(8), key=modulo)
    assert slt.count(9) == 0

def test_append():
    slt = SortedListWithKey(load=4, key=modulo)

    slt.append(0)

    for val in range(1, 10):
        slt.append(val)
        slt._check()

@raises(ValueError)
def test_append_valueerror():
    slt = SortedListWithKey(range(100), key=modulo)
    slt.append(5)

def test_extend():
    slt = SortedListWithKey(load=4, key=modulo)

    slt.extend(range(5))
    slt._check()

    slt.extend(range(6, 10))
    slt._check()

@raises(ValueError)
def test_extend_valueerror1():
    slt = SortedListWithKey(key=modulo)
    slt.extend([1, 2, 3, 5, 4, 6])

@raises(ValueError)
def test_extend_valueerror2():
    slt = SortedListWithKey(range(20), load=4, key=modulo)
    slt.extend([17, 18, 19, 20, 21, 22, 23])

def test_insert():
    slt = SortedListWithKey(range(10), load=4, key=modulo)
    slt.insert(-100, 0)
    slt._check()
    slt.insert(-1, 9)
    slt._check()
    slt.insert(0, 10)
    slt._check()

    slt = SortedListWithKey(load=4, key=modulo)
    slt.insert(0, 5)
    slt._check()

    slt = SortedListWithKey(range(5, 15), load=4, key=modulo)
    for rpt in range(8):
        slt.insert(0, 10)
        slt._check()

    slt = SortedListWithKey(range(10), load=4, key=modulo)
    slt.insert(8, 8)
    slt._check()

@raises(ValueError)
def test_insert_valueerror1():
    slt = SortedListWithKey(range(10), load=4, key=modulo)
    slt.insert(10, 5)

@raises(ValueError)
def test_insert_valueerror2():
    slt = SortedListWithKey(range(10), load=4, key=modulo)
    slt.insert(0, 9)

@raises(ValueError)
def test_insert_valueerror3():
    slt = SortedListWithKey(range(10), load=4, key=modulo)
    slt.insert(5, 3)

@raises(ValueError)
def test_insert_valueerror4():
    slt = SortedListWithKey(range(10), load=4, key=modulo)
    slt.insert(5, 7)

def test_pop():
    slt = SortedListWithKey(range(10), load=4, key=modulo)
    slt._check()
    assert slt.pop() == 9
    slt._check()
    assert slt.pop(0) == 0
    slt._check()
    assert slt.pop(-2) == 7
    slt._check()
    assert slt.pop(4) == 5
    slt._check()

@raises(IndexError)
def test_pop_indexerror1():
    slt = SortedListWithKey(range(10), load=4, key=modulo)
    slt.pop(-11)

@raises(IndexError)
def test_pop_indexerror2():
    slt = SortedListWithKey(range(10), load=4, key=modulo)
    slt.pop(10)

def test_index():
    slt = SortedListWithKey(range(100), load=7, key=modulo)

    for pos, val in enumerate(sorted(range(100), key=modulo)):
        assert val == slt.index(pos)

    assert slt.index(9, 0, 1000) == 90

    slt = SortedListWithKey((0 for rpt in range(100)), load=7, key=modulo)

    for start in range(100):
        for stop in range(start, 100):
            assert slt.index(0, start, stop + 1) == start

    for start in range(100):
        assert slt.index(0, -(100 - start)) == start

    assert slt.index(0, -1000) == 0

@raises(ValueError)
def test_index_valueerror1():
    slt = SortedListWithKey([0] * 10, load=4, key=modulo)
    slt.index(0, 10)

@raises(ValueError)
def test_index_valueerror2():
    slt = SortedListWithKey([0] * 10, load=4, key=modulo)
    slt.index(0, 0, -10)

@raises(ValueError)
def test_index_valueerror3():
    slt = SortedListWithKey([0] * 10, load=4, key=modulo)
    slt.index(0, 7, 3)

@raises(ValueError)
def test_index_valueerror4():
    slt = SortedListWithKey([0] * 10, load=4, key=modulo)
    slt.index(1)

@raises(ValueError)
def test_index_valueerror5():
    slt = SortedListWithKey(key=modulo)
    slt.index(1)

@raises(ValueError)
def test_index_valueerror6():
    slt = SortedListWithKey(range(100), load=4, key=modulo)
    slt.index(91, 0, 15)

@raises(ValueError)
def test_index_valueerror7():
    slt = SortedListWithKey([0] * 10 + [1] * 10 + [2] * 10, load=4, key=modulo)
    slt.index(1, 0, 10)

@raises(ValueError)
def test_index_valueerror8():
    slt = SortedListWithKey(range(10), load=4, key=modulo)
    slt.index(4, 5)

@raises(ValueError)
def test_index_valueerror9():
    slt = SortedListWithKey(load=4, key=modulo)
    slt.index(5)

@raises(ValueError)
def test_index_valueerror10():
    slt = SortedListWithKey(range(10), load=4, key=modulo)
    slt.index(19)

def test_mul():
    this = SortedListWithKey(range(10), load=4, key=modulo)
    that = this * 5
    this._check()
    that._check()
    assert this == sorted(range(10), key=modulo)
    assert that == sorted(list(range(10)) * 5, key=modulo)
    assert this != that

def test_imul():
    this = SortedListWithKey(range(10), load=4, key=modulo)
    this *= 5
    this._check()
    assert this == sorted(list(range(10)) * 5, key=modulo)

def test_op_add():
    this = SortedListWithKey(range(10), load=4, key=modulo)
    assert (this + this + this) == (this * 3)

    that = SortedListWithKey(range(10), load=4, key=modulo)
    that += that
    that += that
    assert that == (this * 4)

def test_eq():
    this = SortedListWithKey(range(10), load=4, key=modulo)
    assert this == list(range(10))
    assert this == tuple(range(10))
    assert not (this == list(range(9)))

def test_ne():
    this = SortedListWithKey(range(10, 20), load=4, key=modulo)
    assert this != list(range(11, 21))
    assert this != tuple(range(10, 21))
    assert this != [0, 1, 2, 3, 3, 5, 6, 7, 8, 9]
    assert this != (val for val in range(10))
    assert this != set()

def test_lt():
    this = SortedListWithKey(range(10, 15), load=4, key=modulo)
    assert this < [10, 11, 13, 13, 14]
    assert this < [10, 11, 12, 13, 14, 15]
    assert this < [11]

def test_le():
    this = SortedListWithKey(range(10, 15), load=4, key=modulo)
    assert this <= [10, 11, 12, 13, 14]
    assert this <= [10, 11, 12, 13, 14, 15]
    assert this <= [10, 11, 13, 13, 14]
    assert this <= [11]

def test_gt():
    this = SortedListWithKey(range(10, 15), load=4, key=modulo)
    assert this > [10, 11, 11, 13, 14]
    assert this > [10, 11, 12, 13]
    assert this > [9]

def test_ge():
    this = SortedListWithKey(range(10, 15), load=4, key=modulo)
    assert this >= [10, 11, 12, 13, 14]
    assert this >= [10, 11, 12, 13]
    assert this >= [10, 11, 11, 13, 14]
    assert this >= [9]

def test_repr():
    this = SortedListWithKey(range(10), load=4, key=modulo)
    assert repr(this).startswith('SortedListWithKey([0, 1, 2, 3, 4, 5, 6, 7, 8, 9], key=<function modulo at ')

def test_repr_recursion():
    this = SortedListWithKey([[1], [2], [3], [4]], key=lambda val: val)
    this._lists[-1].append(this)
    assert repr(this).startswith('SortedListWithKey([[1], [2], [3], [4], ...], key=<function ')

def test_repr_subclass():
    class CustomSortedListWithKey(SortedListWithKey):
        pass
    this = CustomSortedListWithKey(range(10), load=4, key=modulo)
    assert repr(this).startswith('CustomSortedListWithKey([0, 1, 2, 3, 4, 5, 6, 7, 8, 9], key=<function modulo at ')

@raises(AssertionError)
def test_check():
    slt = SortedListWithKey(range(10), load=4, key=modulo)
    slt._len = 5
    slt._check()

if __name__ == '__main__':
    import nose
    nose.main()



# -*- coding: utf-8 -*-

from __future__ import print_function
from sys import hexversion

import copy
import bisect
import random
from .context import sortedcontainers
from sortedcontainers import SortedListWithKey
from nose.tools import raises
from functools import wraps

class SortedList(SortedListWithKey):
    pass

if hexversion < 0x03000000:
    from itertools import izip as zip
    range = xrange

random.seed(0)
actions = []

def frange(start, stop, step):
    while start < stop:
        yield start
        start += step

class actor:
    def __init__(self, count):
        self._count = count
    def __call__(self, func):
        actions.extend([func] * self._count)
        return func

def not_empty(func):
    @wraps(func)
    def wrapper(slt):
        if len(slt) < 100:
            stress_update(slt)
        func(slt)
    return wrapper

@actor(1)
def stress_clear(slt):
    if random.randrange(100) < 10:
        slt.clear()
    else:
        values = list(slt)
        slt.clear()
        slt.update(values[:int(len(values) / 2)])

@actor(1)
def stress_add(slt):
    if random.randrange(100) < 10:
        slt.clear()
    slt.add(random.random())

@actor(1)
def stress_update(slt):
    slt.update((random.random() for rpt in range(350)))

@actor(1)
@not_empty
def stress_contains(slt):
    if random.randrange(100) < 10:
        slt.clear()
        assert 0 not in slt
    else:
        val = slt[random.randrange(len(slt))]
        assert val in slt
        assert 1 not in slt

@actor(1)
@not_empty
def stress_discard(slt):
    val = slt[random.randrange(len(slt))]
    slt.discard(val)

@actor(1)
def stress_discard2(slt):
    if random.randrange(100) < 10:
        slt.clear()
    slt.discard(random.random())

@actor(1)
def stress_remove(slt):
    if len(slt) > 0:
        val = slt[random.randrange(len(slt))]
        slt.remove(val)

    try:
        slt.remove(1)
        assert False
    except ValueError:
        pass

    try:
        slt.remove(-1)
        assert False
    except ValueError:
        pass

@actor(1)
@not_empty
def stress_delitem(slt):
    del slt[random.randrange(len(slt))]

@actor(1)
def stress_getitem(slt):
    if len(slt) > 0:
        pos = random.randrange(len(slt))
        assert slt[pos] == list(slt)[pos]

        try:
            slt[-(slt._len + 5)]
            assert False
        except IndexError:
            pass

        try:
            slt[slt._len + 5]
            assert False
        except IndexError:
            pass
    else:
        try:
            slt[0]
            assert False
        except IndexError:
            pass

@actor(1)
@not_empty
def stress_setitem(slt):
    pos = random.randrange(len(slt))
    slt[pos] = slt[pos]

@actor(1)
@not_empty
def stress_setitem2(slt):
    pos = random.randrange(int(len(slt) / 100)) * 100
    slt[pos] = slt[pos]

@actor(1)
@not_empty
def stress_getset_slice(slt):
    start, stop = sorted(random.randrange(len(slt)) for rpt in range(2))
    step = random.choice([-3, -2, -1, 1, 1, 1, 1, 1, 2, 3])
    lst = slt[start:stop:step]
    assert all(lst[pos - 1] <= lst[pos] for pos in range(1, len(lst)))
    slt[start:stop:step] = lst

@actor(1)
@not_empty
def stress_delitem_slice(slt):
    start, stop = sorted(random.randrange(len(slt)) for rpt in range(2))
    step = random.choice([-3, -2, -1, 1, 1, 1, 1, 1, 2, 3])
    del slt[start:stop:step]

@actor(1)
def stress_iter(slt):
    itr1 = iter(slt)
    itr2 = (slt[pos] for pos in range(len(slt)))
    assert all(tup[0] == tup[1] for tup in zip(itr1, itr2))

@actor(1)
def stress_reversed(slt):
    itr = reversed(list(reversed(slt)))
    assert all(tup[0] == tup[1] for tup in zip(slt, itr))

@actor(1)
def stress_bisect_left(slt):
    values = list(slt)
    value = random.random()
    values.sort()
    assert bisect.bisect_left(values, value) == slt.bisect_left(value)

@actor(1)
def stress_bisect(slt):
    values = list(slt)
    value = random.random()
    values.sort()
    assert bisect.bisect(values, value) == slt.bisect(value)

@actor(1)
def stress_bisect_right(slt):
    values = list(slt)
    value = random.random()
    values.sort()
    assert bisect.bisect_right(values, value) == slt.bisect_right(value)

@actor(1)
@not_empty
def stress_dups(slt):
    pos = min(random.randrange(len(slt)), 300)
    val = slt[pos]
    for rpt in range(pos):
        slt.add(val)

@actor(1)
@not_empty
def stress_count(slt):
    values = list(slt)
    val = slt[random.randrange(len(slt))]
    assert slt.count(val) == values.count(val)

@actor(1)
def stress_append(slt):
    if random.randrange(100) < 10:
        slt.clear()
    if len(slt) == 0:
        slt.append(random.random())
    else:
        slt.append(slt[-1])

@actor(1)
def stress_extend(slt):
    if random.randrange(100) < 10:
        slt.clear()
    if len(slt) == 0:
        slt.extend(float(val) / 1000 for val in range(1000))
    else:
        slt.extend(frange(slt[-1], 1, 0.001))

@actor(1)
@not_empty
def stress_insert(slt):
    slt.insert(0, slt[0])
    slt.insert(-(len(slt) + 10), slt[0])

    slt.insert(len(slt), slt[-1])
    slt.insert(len(slt) + 10, slt[-1])

    pos = random.randrange(len(slt))
    slt.insert(pos, slt[pos])

@actor(1)
def stress_insert2(slt):
    if random.randrange(100) < 10:
        slt.clear()
    if len(slt) == 0:
        slt.insert(0, random.random())
    else:
        values = list(slt)[:250]
        for val in values:
            slt.insert(slt.index(val), val)

@actor(1)
@not_empty
def stress_pop(slt):
    pos = random.randrange(len(slt)) + 1
    assert slt[-pos] == slt.pop(-pos)

@actor(1)
@not_empty
def stress_index(slt):
    values = set(slt)
    slt.clear()
    slt.update(values)
    pos = random.randrange(len(slt))
    assert slt.index(slt[pos]) == pos

@actor(1)
@not_empty
def stress_index2(slt):
    values = list(slt)[:3] * 200
    slt = SortedList(values)
    for idx, val in enumerate(slt):
        assert slt.index(val, idx) == idx

@actor(1)
def stress_mul(slt):
    values = list(slt)
    mult = random.randrange(10)
    values *= mult
    values.sort()
    assert (slt * mult) == values

@actor(1)
def stress_imul(slt):
    mult = random.randrange(10)
    slt *= mult

@actor(1)
@not_empty
def stress_reversed(slt):
    itr = reversed(slt)
    pos = random.randrange(1, len(slt))
    for rpt in range(pos):
        val = next(itr)
    assert val == slt[-pos]

@actor(1)
@not_empty
def stress_eq(slt):
    values = []
    assert not (values == slt)

@actor(1) # Disabled!!!
@not_empty
def stress_lt(slt):
    values = list(slt) # Doesn't work with nose!
    assert not (values < slt)
    values = SortedList(value - 1 for value in values)
    assert values < slt
    values = []
    assert values < slt
    assert not (slt < values)

def test_stress(repeat=1000):
    slt = SortedList((random.random() for rpt in range(1000)))

    for rpt in range(repeat):
        action = random.choice(actions)
        action(slt)

        slt._check()

        while len(slt) > 2000:
            # Shorten the sortedlist. This maintains the "jaggedness"
            # of the sublists which helps coverage.
            pos = random.randrange(len(slt._maxes))
            del slt._maxes[pos]
            del slt._keys[pos]
            del slt._lists[pos]
            slt._len = sum(len(sublist) for sublist in slt._lists)
            slt._index = []
            slt._check()

    slt._check()

    stress_update(slt)

    while len(slt) > 0:
        pos = random.randrange(len(slt))
        del slt[pos]

    slt._check()

if __name__ == '__main__':
    import sys
    from datetime import datetime

    start = datetime.now()

    print('Python', sys.version_info)

    try:
        num = int(sys.argv[1])
        print('Setting iterations to', num)
    except:
        print('Setting iterations to 1000 (default)')
        num = 1000

    try:
        pea = int(sys.argv[2])
        random.seed(pea)
        print('Setting seed to', pea)
    except:
        print('Setting seed to 0 (default)')
        random.seed(0)

    try:
        test_stress(num)
    except:
        raise
    finally:
        print('Exiting after', (datetime.now() - start))



# -*- coding: utf-8 -*-

import random, string
from .context import sortedcontainers
from sortedcontainers import SortedDict
from nose.tools import raises
from sys import hexversion

if hexversion < 0x03000000:
    range = xrange

def negate(value):
    return -value

def modulo(value):
    return value % 10

def get_keysview(dic):
    if hexversion < 0x03000000:
        return dic.viewkeys()
    else:
        return dic.keys()

def get_valuesview(dic):
    if hexversion < 0x03000000:
        return dic.viewvalues()
    else:
        return dic.values()

def get_itemsview(dic):
    if hexversion < 0x03000000:
        return dic.viewitems()
    else:
        return dic.items()

def test_init():
    temp = SortedDict()
    temp._check()

def test_init_args():
    temp = SortedDict([('a', 1), ('b', 2)])
    assert len(temp) == 2
    assert temp['a'] == 1
    assert temp['b'] == 2
    temp._check()

def test_init_kwargs():
    temp = SortedDict(a=1, b=2)
    assert len(temp) == 2
    assert temp['a'] == 1
    assert temp['b'] == 2
    temp._check()

def test_clear():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert len(temp) == 26
    assert list(temp.items()) == mapping
    temp.clear()
    assert len(temp) == 0

def test_contains():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert all((val in temp) for val in string.ascii_lowercase)

def test_delitem():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    del temp['a']
    temp._check()

def test_getitem():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert all((temp[val] == pos) for pos, val in enumerate(string.ascii_lowercase))

def test_eq():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp1 = SortedDict(mapping)
    temp2 = SortedDict(mapping)
    assert temp1 == temp2
    assert not (temp1 != temp2)
    temp2['a'] = 100
    assert temp1 != temp2
    assert not (temp1 == temp2)
    del temp2['a']
    assert temp1 != temp2
    assert not (temp1 == temp2)
    temp2['zz'] = 0
    assert temp1 != temp2
    assert not (temp1 == temp2)

def test_iter():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert all(lhs == rhs for lhs, rhs in zip(temp, string.ascii_lowercase))

def test_iter_key():
    temp = SortedDict(negate, 7, ((val, val) for val in range(100)))
    assert all(lhs == rhs for lhs, rhs in zip(temp, reversed(range(100))))

def test_reversed():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert all(lhs == rhs for lhs, rhs in
               zip(reversed(temp), reversed(string.ascii_lowercase)))

def test_reversed_key():
    temp = SortedDict(modulo, 7, ((val, val) for val in range(100)))
    values = sorted(range(100), key=modulo)
    assert all(lhs == rhs for lhs, rhs in zip(reversed(temp), reversed(values)))

def test_islice():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(7, mapping)

    for start in range(30):
        for stop in range(30):
            assert list(temp.islice(start, stop)) == list(string.ascii_lowercase[start:stop])

def test_irange():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(7, mapping)
    for start in range(26):
        for stop in range(start + 1, 26):
            result = list(string.ascii_lowercase[start:stop])
            assert list(temp.irange(result[0], result[-1])) == result

def test_irange_key():
    temp = SortedDict(modulo, 7, ((val, val) for val in range(100)))
    values = sorted(range(100), key=modulo)

    for start in range(10):
        for stop in range(start, 10):
            result = list(temp.irange_key(start, stop))
            assert result == values[(start * 10):((stop + 1) * 10)]

def test_len():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert len(temp) == 26

def test_setitem():
    temp = SortedDict()

    for pos, key in enumerate(string.ascii_lowercase):
        temp[key] = pos
        temp._check()

    assert len(temp) == 26

    for pos, key in enumerate(string.ascii_lowercase):
        temp[key] = pos
        temp._check()

    assert len(temp) == 26

def test_copy():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    dup = temp.copy()
    assert len(temp) == 26
    assert len(dup) == 26
    dup.clear()
    assert len(temp) == 26
    assert len(dup) == 0

def test_copy_copy():
    import copy
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    dup = copy.copy(temp)
    assert len(temp) == 26
    assert len(dup) == 26
    dup.clear()
    assert len(temp) == 26
    assert len(dup) == 0

def test_fromkeys():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict.fromkeys(mapping, 1)
    assert all(temp[key] == 1 for key in temp)

def test_get():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert temp.get('a') == 0
    assert temp.get('A', -1) == -1

def test_has_key():
    if hexversion > 0x03000000:
        return
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert temp.has_key('a')

def test_items():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert list(temp.items()) == mapping

def test_iteritems():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert list(temp.iteritems()) == mapping

def test_keys():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert list(temp.keys()) == [key for key, pos in mapping]

def test_iterkeys():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert list(temp.iterkeys()) == [key for key, pos in mapping]

def test_values():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert list(temp.values()) == [pos for key, pos in mapping]

def test_itervalues():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert list(temp.itervalues()) == [pos for key, pos in mapping]

def test_pop():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert temp.pop('a') == 0
    assert temp.pop('a', -1) == -1

@raises(KeyError)
def test_pop2():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    temp.pop('A')

def test_popitem():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert temp.popitem() == ('z', 25)

@raises(KeyError)
def test_popitem2():
    temp = SortedDict()
    temp.popitem()

def test_popitem3():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert temp.popitem(last=False) == ('a', 0)

def test_setdefault():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert temp.setdefault('a', -1) == 0
    assert temp['a'] == 0
    assert temp.setdefault('A', -1) == -1

def test_update():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict()
    temp.update()
    temp.update(mapping)
    temp.update(dict(mapping))
    temp.update(mapping[5:7])
    assert list(temp.items()) == mapping

def test_update2():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict()
    temp.update(**dict(mapping))
    assert list(temp.items()) == mapping

def test_repr():
    temp = SortedDict({'alice': 3, 'bob': 1, 'carol': 2, 'dave': 4})
    assert repr(temp) == "SortedDict(None, 1000, {'alice': 3, 'bob': 1, 'carol': 2, 'dave': 4})"

def test_repr_recursion():
    temp = SortedDict({'alice': 3, 'bob': 1, 'carol': 2, 'dave': 4})
    temp['bob'] = temp
    assert repr(temp) == "SortedDict(None, 1000, {'alice': 3, 'bob': ..., 'carol': 2, 'dave': 4})"

def test_repr_subclass():
    class CustomSortedDict(SortedDict):
        pass
    temp = CustomSortedDict({'alice': 3, 'bob': 1, 'carol': 2, 'dave': 4})
    assert repr(temp) == "CustomSortedDict(None, 1000, {'alice': 3, 'bob': 1, 'carol': 2, 'dave': 4})"

def test_iloc():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert temp.iloc[0] == 'a'
    assert temp.iloc[-1] == 'z'
    assert temp.iloc[-3:] == ['x', 'y', 'z']
    del temp.iloc[0]
    assert temp.iloc[0] == 'b'
    del temp.iloc[-3:]
    assert temp.iloc[-1] == 'w'

def test_index():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert temp.index('a') == 0
    assert temp.index('f', 3, -3) == 5

def test_index_key():
    temp = SortedDict(negate, 7, ((val, val) for val in range(100)))
    assert all(temp.index(val) == (99 - val) for val in range(100))

def test_bisect():
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    assert temp.bisect_left('a') == 0
    assert temp.bisect_right('f') == 6
    assert temp.bisect('f') == 6

def test_bisect_key():
    temp = SortedDict(modulo, 7, ((val, val) for val in range(100)))
    assert all(temp.bisect(val) == ((val % 10) + 1) * 10 for val in range(100))
    assert all(temp.bisect_right(val) == ((val % 10) + 1) * 10 for val in range(100))
    assert all(temp.bisect_left(val) == (val % 10) * 10 for val in range(100))

def test_bisect_key2():
    temp = SortedDict(modulo, 7, ((val, val) for val in range(100)))
    assert all(temp.bisect_key(val) == ((val % 10) + 1) * 10 for val in range(10))
    assert all(temp.bisect_key_right(val) == ((val % 10) + 1) * 10 for val in range(10))
    assert all(temp.bisect_key_left(val) == (val % 10) * 10 for val in range(10))

def test_keysview():
    if hexversion < 0x02070000: return

    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping[:13])
    keys = get_keysview(temp)

    assert len(keys) == 13
    assert 'a' in keys
    assert list(keys) == [val for val, pos in mapping[:13]]
    assert keys[0] == 'a'
    assert list(reversed(keys)) == list(reversed(string.ascii_lowercase[:13]))
    assert keys.index('f') == 5
    assert keys.count('m') == 1
    assert keys.count('0') == 0
    assert keys.isdisjoint(['1', '2', '3'])

    temp.update(mapping[13:])

    assert len(keys) == 26
    assert 'z' in keys
    assert list(keys) == [val for val, pos in mapping]

    that = dict(mapping)

    that_keys = get_keysview(that)

    assert keys == that_keys
    assert not (keys != that_keys)
    assert not (keys < that_keys)
    assert not (keys > that_keys)
    assert keys <= that_keys
    assert keys >= that_keys

    assert list(keys & that_keys) == [val for val, pos in mapping]
    assert list(keys | that_keys) == [val for val, pos in mapping]
    assert list(keys - that_keys) == []
    assert list(keys ^ that_keys) == []

    keys = get_keysview(SortedDict(mapping[:2]))
    assert repr(keys) == "SortedDict_keys(['a', 'b'])"

def test_valuesview():
    if hexversion < 0x02070000: return

    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping[:13])
    values = get_valuesview(temp)

    assert len(values) == 13
    assert 0 in values
    assert list(values) == [pos for val, pos in mapping[:13]]
    assert values[0] == 0
    assert values[-3:] == [10, 11, 12]
    assert list(reversed(values)) == list(reversed(range(13)))
    assert values.index(5) == 5
    assert values.count(10) == 1

    temp.update(mapping[13:])

    assert len(values) == 26
    assert 25 in values
    assert list(values) == [pos for val, pos in mapping]

    that = dict(mapping)
    that_values = get_valuesview(that)

    values = get_valuesview(SortedDict(mapping[:2]))
    assert repr(values) == "SortedDict_values([0, 1])"

@raises(ValueError)
def test_valuesview():
    if hexversion < 0x02070000: raise ValueError

    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping[:13])
    values = get_valuesview(temp)

    values.index(13)

@raises(TypeError)
def test_values_view_lt():
    if hexversion < 0x02070000: raise TypeError
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    values = get_valuesview(temp)
    that = dict(mapping)
    that_values = get_valuesview(that)
    values < that_values

@raises(TypeError)
def test_values_view_gt():
    if hexversion < 0x02070000: raise TypeError
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    values = get_valuesview(temp)
    that = dict(mapping)
    that_values = get_valuesview(that)
    values > that_values

@raises(TypeError)
def test_values_view_le():
    if hexversion < 0x02070000: raise TypeError
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    values = get_valuesview(temp)
    that = dict(mapping)
    that_values = get_valuesview(that)
    values <= that_values

@raises(TypeError)
def test_values_view_ge():
    if hexversion < 0x02070000: raise TypeError
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    values = get_valuesview(temp)
    that = dict(mapping)
    that_values = get_valuesview(that)
    values >= that_values

@raises(TypeError)
def test_values_view_and():
    if hexversion < 0x02070000: raise TypeError
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    values = get_valuesview(temp)
    that = dict(mapping)
    that_values = get_valuesview(that)
    values & that_values

@raises(TypeError)
def test_values_view_or():
    if hexversion < 0x02070000: raise TypeError
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    values = get_valuesview(temp)
    that = dict(mapping)
    that_values = get_valuesview(that)
    values | that_values

@raises(TypeError)
def test_values_view_sub():
    if hexversion < 0x02070000: raise TypeError
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    values = get_valuesview(temp)
    that = dict(mapping)
    that_values = get_valuesview(that)
    values - that_values

@raises(TypeError)
def test_values_view_xor():
    if hexversion < 0x02070000: raise TypeError
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping)
    values = get_valuesview(temp)
    that = dict(mapping)
    that_values = get_valuesview(that)
    values ^ that_values

def test_itemsview():
    if hexversion < 0x02070000: return
    mapping = [(val, pos) for pos, val in enumerate(string.ascii_lowercase)]
    temp = SortedDict(mapping[:13])
    items = get_itemsview(temp)

    assert len(items) == 13
    assert ('a', 0) in items
    assert list(items) == mapping[:13]
    assert items[0] == ('a', 0)
    assert items[-3:] == [('k', 10), ('l', 11), ('m', 12)]
    assert list(reversed(items)) == list(reversed(mapping[:13]))
    assert items.index(('f', 5)) == 5
    assert items.count(('m', 12)) == 1
    assert items.isdisjoint([('0', 26), ('1', 27)])
    assert not items.isdisjoint([('a', 0), ('b', 1)])

    temp.update(mapping[13:])

    assert len(items) == 26
    assert ('z', 25) in items
    assert list(items) == mapping

    that = dict(mapping)

    that_items = get_itemsview(that)

    assert items == that_items
    assert not (items != that_items)
    assert not (items < that_items)
    assert not (items > that_items)
    assert items <= that_items
    assert items >= that_items

    assert list(items & that_items) == mapping
    assert list(items | that_items) == mapping
    assert list(items - that_items) == []
    assert list(items ^ that_items) == []

    items = SortedDict(mapping[:2]).viewitems()
    assert repr(items) == "SortedDict_items([('a', 0), ('b', 1)])"

def test_pickle():
    import pickle
    alpha = SortedDict(negate, 500, zip(range(10000), range(10000)))
    beta = pickle.loads(pickle.dumps(alpha))
    assert alpha == beta
    assert alpha._key == beta._key
    assert alpha._load == beta._load

if __name__ == '__main__':
    import nose
    nose.main()



# -*- coding: utf-8 -*-

from sys import hexversion

import random
from .context import sortedcontainers
from sortedcontainers import SortedSet
from nose.tools import raises

if hexversion < 0x03000000:
    range = xrange

def negate(value):
    return -value

def modulo(value):
    return value % 10

def test_init():
    temp = SortedSet(range(100), load=7)
    temp._check()
    assert all(val == temp[val] for val in temp)

def test_contains():
    temp = SortedSet(range(100), load=7)
    assert all(val in temp for val in range(100))
    assert all(val not in temp for val in range(100, 200))

def test_getitem():
    temp = SortedSet(range(100), load=7)
    assert all(val == temp[val] for val in temp)

def test_getitem_slice():
    vals = list(range(100))
    temp = SortedSet(vals, load=7)
    assert temp[20:30] == vals[20:30]

def test_getitem_key():
    temp = SortedSet(range(100), load=7, key=negate)
    assert all(temp[val] == (99 - val) for val in range(100))

def test_delitem():
    temp = SortedSet(range(100), load=7)
    for val in reversed(range(50)):
        del temp[val]
    assert all(temp[pos] == (pos + 50) for pos in range(50))

def test_delitem_slice():
    vals = list(range(100))
    temp = SortedSet(vals, load=7)
    del vals[20:40:2]
    del temp[20:40:2]
    assert temp == set(vals)

def test_delitem_key():
    temp = SortedSet(range(100), load=7, key=modulo)
    values = sorted(range(100), key=modulo)
    for val in range(10):
        del temp[val]
        del values[val]
    assert list(temp) == list(values)

def test_eq():
    alpha = SortedSet(range(100), load=7)
    beta = SortedSet(range(100), load=17)
    assert alpha == beta
    assert alpha == beta._set
    beta.add(101)
    assert not (alpha == beta)

def test_ne():
    alpha = SortedSet(range(100), load=7)
    beta = SortedSet(range(99), load=17)
    assert alpha != beta
    beta.add(100)
    assert alpha != beta
    assert alpha != beta._set
    assert alpha != list(range(101))

def test_lt_gt():
    temp = SortedSet(range(100), load=7)
    that = SortedSet(range(25, 75), load=9)
    assert that < temp
    assert not (temp < that)
    assert that < temp._set
    assert temp > that
    assert not (that > temp)
    assert temp > that._set

def test_le_ge():
    alpha = SortedSet(range(100), load=7)
    beta = SortedSet(range(101), load=17)
    assert alpha <= beta
    assert not (beta <= alpha)
    assert alpha <= beta._set
    assert beta >= alpha
    assert not (alpha >= beta)
    assert beta >= alpha._set

def test_iter():
    temp = SortedSet(range(100), load=7)
    assert all(val == temp[val] for val in iter(temp))

def test_reversed():
    temp = SortedSet(range(100), load=7)
    assert all(val == temp[val] for val in reversed(temp))

def test_islice():
    ss = SortedSet(load=7)

    assert [] == list(ss.islice())

    values = list(range(53))
    ss.update(values)

    for start in range(53):
        for stop in range(53):
            assert list(ss.islice(start, stop)) == values[start:stop]

    for start in range(53):
        for stop in range(53):
            assert list(ss.islice(start, stop, reverse=True)) == values[start:stop][::-1]

    for start in range(53):
        assert list(ss.islice(start=start)) == values[start:]
        assert list(ss.islice(start=start, reverse=True)) == values[start:][::-1]

    for stop in range(53):
        assert list(ss.islice(stop=stop)) == values[:stop]
        assert list(ss.islice(stop=stop, reverse=True)) == values[:stop][::-1]

def test_irange():
    ss = SortedSet(load=7)

    assert [] == list(ss.irange())

    values = list(range(53))
    ss.update(values)

    for start in range(53):
        for end in range(start, 53):
            assert list(ss.irange(start, end)) == values[start:(end + 1)]
            assert list(ss.irange(start, end, reverse=True)) == values[start:(end + 1)][::-1]

    for start in range(53):
        for end in range(start, 53):
            assert list(range(start, end)) == list(ss.irange(start, end, (True, False)))

    for start in range(53):
        for end in range(start, 53):
            assert list(range(start + 1, end + 1)) == list(ss.irange(start, end, (False, True)))

    for start in range(53):
        for end in range(start, 53):
            assert list(range(start + 1, end)) == list(ss.irange(start, end, (False, False)))

    for start in range(53):
        assert list(range(start, 53)) == list(ss.irange(start))

    for end in range(53):
        assert list(range(0, end)) == list(ss.irange(None, end, (True, False)))

    assert values == list(ss.irange(inclusive=(False, False)))

    assert [] == list(ss.irange(53))
    assert values == list(ss.irange(None, 53, (True, False)))

def test_irange_key():
    values = sorted(range(100), key=modulo)

    for load in range(5, 16):
        ss = SortedSet(range(100), load=load, key=modulo)

        for start in range(10):
            for end in range(start, 10):
                temp = list(ss.irange_key(start, end))
                assert temp == values[(start * 10):((end + 1) * 10)]

                temp = list(ss.irange_key(start, end, reverse=True))
                assert temp == values[(start * 10):((end + 1) * 10)][::-1]

        for start in range(10):
            for end in range(start, 10):
                temp = list(ss.irange_key(start, end, inclusive=(True, False)))
                assert temp == values[(start * 10):(end * 10)]

        for start in range(10):
            for end in range(start, 10):
                temp = list(ss.irange_key(start, end, (False, True)))
                assert temp == values[((start + 1) * 10):((end + 1) * 10)]

        for start in range(10):
            for end in range(start, 10):
                temp = list(ss.irange_key(start, end, inclusive=(False, False)))
                assert temp == values[((start + 1) * 10):(end * 10)]

        for start in range(10):
            temp = list(ss.irange_key(min_key=start))
            assert temp == values[(start * 10):]

        for end in range(10):
            temp = list(ss.irange_key(max_key=end))
            assert temp == values[:(end + 1) * 10]

def test_len():
    temp = SortedSet(range(100), load=7)
    assert len(temp) == 100

def test_add():
    temp = SortedSet(range(100), load=7)
    temp.add(100)
    temp.add(90)
    temp._check()
    assert all(val == temp[val] for val in range(101))

def test_bisect():
    temp = SortedSet(range(100), load=7)
    assert all(temp.bisect_left(val) == val for val in range(100))
    assert all(temp.bisect(val) == (val + 1) for val in range(100))
    assert all(temp.bisect_right(val) == (val + 1) for val in range(100))

def test_bisect_key():
    temp = SortedSet(range(100), key=lambda val: val, load=7)
    assert all(temp.bisect_key_left(val) == val for val in range(100))
    assert all(temp.bisect_key(val) == (val + 1) for val in range(100))
    assert all(temp.bisect_key_right(val) == (val + 1) for val in range(100))

def test_clear():
    temp = SortedSet(range(100), load=7)
    temp.clear()
    temp._check()
    assert len(temp) == 0

def test_copy():
    temp = SortedSet(range(100), load=7)
    that = temp.copy()
    that.add(1000)
    assert len(temp) == 100
    assert len(that) == 101

def test_copy_copy():
    import copy
    temp = SortedSet(range(100), load=7)
    that = copy.copy(temp)
    that.add(1000)
    assert len(temp) == 100
    assert len(that) == 101

def test_count():
    temp = SortedSet(range(100), load=7)
    assert all(temp.count(val) == 1 for val in range(100))
    assert temp.count(100) == 0
    assert temp.count(0) == 1
    temp.add(0)
    assert temp.count(0) == 1
    temp._check()

def test_sub():
    temp = SortedSet(range(100), load=7)
    that = temp - range(0, 10) - range(10, 20)
    assert all(val == temp[val] for val in range(100))
    assert all((val + 20) == that[val] for val in range(80))

def test_difference():
    temp = SortedSet(range(100), load=7)
    that = temp.difference(range(0, 10), range(10, 20))
    assert all(val == temp[val] for val in range(100))
    assert all((val + 20) == that[val] for val in range(80))

def test_difference_update():
    temp = SortedSet(range(100), load=7)
    temp.difference_update(range(0, 10), range(10, 20))
    assert all((val + 20) == temp[val] for val in range(80))

def test_isub():
    temp = SortedSet(range(100), load=7)
    temp -= range(0, 10)
    temp -= range(10, 20)
    assert all((val + 20) == temp[val] for val in range(80))

def test_discard():
    temp = SortedSet(range(100), load=7)
    temp.discard(0)
    temp.discard(99)
    temp.discard(50)
    temp.discard(1000)
    temp._check()
    assert len(temp) == 97

def test_index():
    temp = SortedSet(range(100), load=7)
    assert all(temp.index(val) == val for val in range(100))

def test_and():
    temp = SortedSet(range(100), load=7)
    that = temp & range(20) & range(10, 30)
    assert all(that[val] == (val + 10) for val in range(10))
    assert all(temp[val] == val for val in range(100))

def test_intersection():
    temp = SortedSet(range(100), load=7)
    that = temp.intersection(range(0, 20), range(10, 30))
    assert all(that[val] == (val + 10) for val in range(10))
    assert all(temp[val] == val for val in range(100))

def test_intersection_update():
    temp = SortedSet(range(100), load=7)
    temp &= range(0, 20)
    temp &= range(10, 30)
    assert all(temp[val] == (val + 10) for val in range(10))

def test_isdisjoint():
    temp = SortedSet(range(100), load=7)
    that = SortedSet(range(100, 200), load=9)
    assert temp.isdisjoint(that)

def test_issubset():
    temp = SortedSet(range(100), load=7)
    that = SortedSet(range(25, 75), load=9)
    assert that.issubset(temp)

def test_issuperset():
    temp = SortedSet(range(100), load=7)
    that = SortedSet(range(25, 75), load=9)
    assert temp.issuperset(that)

def test_xor():
    temp = SortedSet(range(0, 75), load=7)
    that = SortedSet(range(25, 100), load=9)
    result = temp ^ that
    assert all(result[val] == val for val in range(25))
    assert all(result[val + 25] == (val + 75) for val in range(25))
    assert all(temp[val] == val for val in range(75))
    assert all(that[val] == (val + 25) for val in range(75))

def test_symmetric_difference():
    temp = SortedSet(range(0, 75), load=7)
    that = SortedSet(range(25, 100), load=9)
    result = temp.symmetric_difference(that)
    assert all(result[val] == val for val in range(25))
    assert all(result[val + 25] == (val + 75) for val in range(25))
    assert all(temp[val] == val for val in range(75))
    assert all(that[val] == (val + 25) for val in range(75))

def test_symmetric_difference_update():
    temp = SortedSet(range(0, 75), load=7)
    that = SortedSet(range(25, 100), load=9)
    temp ^= that
    assert all(temp[val] == val for val in range(25))
    assert all(temp[val + 25] == (val + 75) for val in range(25))

def test_pop():
    temp = SortedSet(range(0, 100), load=7)
    temp.pop()
    temp.pop(0)
    assert all(temp[val] == (val + 1) for val in range(98))

def test_remove():
    temp = SortedSet(range(0, 100), load=7)
    temp.remove(50)

def test_or():
    temp = SortedSet(range(0, 50), load=7)
    that = SortedSet(range(50, 100), load=9)
    result = temp | that
    assert all(result[val] == val for val in range(100))
    assert all(temp[val] == val for val in range(50))
    assert all(that[val] == (val + 50) for val in range(50))

def test_union():
    temp = SortedSet(range(0, 50), load=7)
    that = SortedSet(range(50, 100), load=9)
    result = temp.union(that)
    assert all(result[val] == val for val in range(100))
    assert all(temp[val] == val for val in range(50))
    assert all(that[val] == (val + 50) for val in range(50))

def test_update():
    temp = SortedSet(range(0, 80), load=7)
    temp.update(range(80, 90), range(90, 100))
    assert all(temp[val] == val for val in range(100))

def test_ior():
    temp = SortedSet(range(0, 80), load=7)
    temp |= range(80, 90)
    temp |= range(90, 100)
    assert all(temp[val] == val for val in range(100))

def test_repr():
    temp = SortedSet(range(0, 10), load=7)
    assert repr(temp) == 'SortedSet([0, 1, 2, 3, 4, 5, 6, 7, 8, 9], key=None, load=7)'

def test_repr_recursion():
    class HashableSortedSet(SortedSet):
        def __hash__(self):
            return hash(tuple(self))

    temp = HashableSortedSet([HashableSortedSet([1]), HashableSortedSet([1, 2])])
    temp.add(temp)
    assert repr(temp) == 'HashableSortedSet([HashableSortedSet([1], key=None, load=1000), HashableSortedSet([1, 2], key=None, load=1000), ...], key=None, load=1000)'

def test_pickle():
    import pickle
    alpha = SortedSet(range(10000), key=negate, load=500)
    beta = pickle.loads(pickle.dumps(alpha))
    assert alpha == beta
    assert alpha._key == beta._key
    assert alpha._load == beta._load

if __name__ == '__main__':
    import nose
    nose.main()



# -*- coding: utf-8 -*-

from __future__ import print_function
from sys import hexversion

import random
from .context import sortedcontainers
from sortedcontainers import SortedSet
from nose.tools import raises
from functools import wraps
import operator

if hexversion < 0x03000000:
    from itertools import izip as zip
    range = xrange

random.seed(0)
actions = []

def actor(func):
    actions.append(func)
    return func

def test_init():
    sst = SortedSet()
    sst._check()

    sst = SortedSet(load=10000)
    assert sst._list._load == 10000
    assert sst._list._twice == 20000
    assert sst._list._half == 5000
    sst._check()

    sst = SortedSet(range(10000))
    assert all(tup[0] == tup[1] for tup in zip(sst, range(10000)))

    sst.clear()
    assert len(sst) == 0
    assert list(iter(sst)) == []
    sst._check()

@actor
def stress_contains(sst):
    values = list(sst)
    assert all((val in sst) for val in values)

@actor
def stress_delitem(sst):
    for rpt in range(100):
        pos = random.randrange(0, len(sst))
        del sst[pos]

@actor
def stress_operator(sst):
    other = SortedSet(sst)
    stress_delitem(other)
    assert other < sst
    assert sst > other

@actor
def stress_getitem(sst):
    other = list(sst)
    assert all(sst[pos] == other[pos] for pos in range(len(sst)))
    
@actor
def stress_reversed(sst):
    other = list(reversed(list(sst)))
    assert all(tup[0] == tup[1] for tup in zip(reversed(sst), other))
    
@actor
def stress_add(sst):
    for rpt in range(100):
        val = random.randrange(0, 1000)
        sst.add(val)

@actor
def stress_count(sst):
    for val in sst:
        assert sst.count(val) == 1

@actor
def stress_difference(sst):
    copy_one = sst.copy()
    stress_delitem(copy_one)
    copy_two = sst.copy()
    stress_delitem(copy_two)
    sst.difference_update(copy_one, copy_two)

@actor
def stress_discard(sst):
    for rpt in range(100):
        pos = random.randrange(0, len(sst))
        val = sst[pos]
        sst.discard(val)

@actor
def stress_index(sst):
    for rpt in range(100):
        pos = random.randrange(0, len(sst))
        val = sst[pos]
        assert pos == sst.index(val)

@actor
def stress_intersection(sst):
    copy_one = sst.copy()
    stress_delitem(copy_one)
    copy_two = sst.copy()
    stress_delitem(copy_two)
    sst.intersection_update(copy_one, copy_two)

@actor
def stress_symmetric_difference(sst):
    copy_one = sst.copy()
    stress_delitem(copy_one)
    sst.symmetric_difference_update(copy_one)

@actor
def stress_pop(sst):
    val = sst[-1]
    assert val == sst.pop()
    for rpt in range(100):
        pos = random.randrange(0, len(sst))
        val = sst[pos]
        assert val == sst.pop(pos)

@actor
def stress_remove(sst):
    for rpt in range(100):
        pos = random.randrange(0, len(sst))
        val = sst[pos]
        sst.remove(val)

@actor
def stress_update(sst):
    def iter_randomly(start, stop, count):
        for rpt in range(count):
            yield random.randrange(start, stop)
    sst.update(iter_randomly(0, 500, 100),
               iter_randomly(500, 1000, 100),
               iter_randomly(1000, 1500, 100),
               iter_randomly(1500, 2000, 100))

@actor
def stress_isdisjoint(sst):
    values = [-1, -2, -3]
    assert sst.isdisjoint(values)

@actor
def stress_issubset(sst):
    that = SortedSet(sst)
    that.update(range(1000))
    assert sst.issubset(that)

@actor
def stress_issuperset(sst):
    that = SortedSet(sst)
    assert sst.issuperset(that)

def test_stress(repeat=1000):
    sst = SortedSet(range(1000))

    for rpt in range(repeat):
        action = random.choice(actions)
        action(sst)

        try:
            sst._check()
        except AssertionError:
            print(action)
            raise

        start_len = len(sst)

        while len(sst) < 500:
            sst.add(random.randrange(0, 2000))

        while len(sst) > 2000:
            del sst[random.randrange(0, len(sst))]

        if start_len != len(sst):
            sst._check()

if __name__ == '__main__':
    import sys
    from datetime import datetime

    start = datetime.now()

    print('Python', sys.version_info)

    try:
        num = int(sys.argv[1])
        print('Setting iterations to', num)
    except:
        print('Setting iterations to 1000 (default)')
        num = 1000

    try:
        pea = int(sys.argv[2])
        random.seed(pea)
        print('Setting seed to', pea)
    except:
        print('Setting seed to 0 (default)')
        random.seed(0)

    try:
        test_stress(num)
    except:
        raise
    finally:
        print('Exiting after', (datetime.now() - start))



# -*- coding: utf-8 -*-

import sys
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

import sortedcontainers

class Tox(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True
    def run_tests(self):
        import tox
        errno = tox.cmdline(self.test_args)
        sys.exit(errno)

with open('README.rst') as fptr:
    readme = fptr.read()

with open('LICENSE') as fptr:
    license = fptr.read()

setup(
    name=sortedcontainers.__title__,
    version=sortedcontainers.__version__,
    description='Python Sorted Container Types: SortedList, SortedDict, and SortedSet',
    long_description=readme,
    author='Grant Jenks',
    author_email='contact@grantjenks.com',
    url='http://www.grantjenks.com/docs/sortedcontainers/',
    license=license,
    packages=find_packages(exclude=('tests', 'docs')),
    tests_require=['tox'],
    cmdclass={'test': Tox},
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ),
)



# -*- coding: utf-8 -*-

from __future__ import print_function
from sys import hexversion

import random
from .context import sortedcontainers
from sortedcontainers import SortedDict
from nose.tools import raises
from functools import wraps

if hexversion < 0x03000000:
    from itertools import izip as zip
    range = xrange

random.seed(0)
actions = []

def actor(func):
    actions.append(func)
    return func

def test_init():
    sdict = SortedDict()
    sdict._check()

    sdict = SortedDict(load=17)
    sdict._check()

    sdict = SortedDict((val, -val) for val in range(10000))
    sdict._check()
    assert all(key == -val for key, val in sdict.iteritems())

    sdict.clear()
    sdict._check()
    assert len(sdict) == 0

    sdict = SortedDict.fromkeys(range(1000), None)
    assert all(sdict[key] == None for key in range(1000))

@actor
def stress_contains(sdict):
    keys = list(sdict)
    assert all((key in sdict for key in keys))

@actor
def stress_delitem(sdict):
    keys = list(sdict)
    for rpt in range(100):
        pos = random.randrange(0, len(sdict))
        del sdict[keys[pos]]
        del keys[pos]

@actor
def stress_getitem(sdict):
    items = list(sdict.iteritems())
    assert all(sdict[key] == value for key, value in items)
    
@actor
def stress_eq(sdict):
    that = dict((key, value) for key, value in sdict.iteritems())
    assert sdict == that

@actor
def stress_setitem_len(sdict):
    start_len = len(sdict)
    keys = list(range(100))
    missing = sum(1 for val in keys if val not in sdict)
    for val in keys:
        sdict[val] = -val
    end_len = len(sdict)
    assert (start_len + missing) == end_len

@actor
def stress_copy(sdict):
    that = sdict.copy()

@actor
def stress_get(sdict):
    keys = list(range(100))
    for key in keys:
        if key in sdict:
            assert sdict.get(key, 1) == -key
        else:
            assert sdict.get(key, 1) == 1

@actor
def stress_has_key(sdict):
    if hexversion > 0x03000000:
        return
    keys = list(range(100))
    for key in keys:
        assert all((key in sdict) == (sdict.has_key(key)) for key in sdict)

@actor
def stress_items_keys_values(sdict):
    items = sdict.items()
    keys = sdict.keys()
    values = sdict.values()
    assert list(items) == list(zip(keys, values))

@actor
def stress_iter_items_keys_values(sdict):
    it = zip(sdict.iteritems(), sdict.iterkeys(), sdict.itervalues())
    assert all(tup[0] == (tup[1], tup[2]) for tup in it)

@actor
def stress_pop(sdict):
    keys = list(range(200))
    for key in keys:
        if key in sdict:
            val = sdict[key]
            assert sdict.pop(key, 1) == val
        else:
            assert sdict.pop(key, 1) == 1

@actor
def stress_popitem(sdict):
    items = [sdict.popitem() for rpt in range(100)]
    keys = [item[0] for item in items]
    assert all(keys[pos - 1] > keys[pos] for pos in range(1, len(keys)))
    assert all(key == -value for key, value in items)

@actor
def stress_setdefault(sdict):
    keys = list(range(200))
    for key in keys:
        if key in sdict:
            assert sdict.setdefault(key) == -key
        else:
            sdict.setdefault(key)
            assert sdict[key] == None
            del sdict[key]

def test_stress(repeat=1000):
    sdict = SortedDict((val, -val) for val in range(1000))

    for rpt in range(repeat):
        action = random.choice(actions)
        action(sdict)

        try:
            sdict._check()
        except AssertionError:
            print(action)
            raise

        start_len = len(sdict)

        while len(sdict) < 500:
            key = random.randrange(0, 2000)
            sdict[key] = -key

        while len(sdict) > 2000:
            key = random.randrange(0, 2000)
            if key in sdict:
                del sdict[key]

        if start_len != len(sdict):
            sdict._check()

if __name__ == '__main__':
    import sys
    from datetime import datetime

    start = datetime.now()

    print('Python', sys.version_info)

    try:
        num = int(sys.argv[1])
        print('Setting iterations to', num)
    except:
        print('Setting iterations to 1000 (default)')
        num = 1000

    try:
        pea = int(sys.argv[2])
        random.seed(pea)
        print('Setting seed to', pea)
    except:
        print('Setting seed to 0 (default)')
        random.seed(0)

    try:
        test_stress(num)
    except:
        raise
    finally:
        print('Exiting after', (datetime.now() - start))



# -*- coding: utf-8 -*-

from sys import hexversion

import random
from .context import sortedcontainers
from sortedcontainers import SortedListWithKey
from itertools import chain
from nose.tools import raises

if hexversion < 0x03000000:
    from itertools import izip as zip
    range = xrange

def negate(val):
    return -val

def test_identity():
    slt = SortedListWithKey(range(100), load=7)
    slt._check()

def test_init():
    slt = SortedListWithKey(key=negate)
    slt._check()

    slt = SortedListWithKey(load=10000, key=negate)
    assert slt._load == 10000
    assert slt._twice == 20000
    assert slt._half == 5000
    slt._check()

    slt = SortedListWithKey(range(10000), key=negate)
    assert all(tup[0] == tup[1] for tup in zip(slt, reversed(range(10000))))

    slt.clear()
    assert slt._len == 0
    assert slt._maxes == []
    assert slt._lists == []
    slt._check()

def test_key():
    slt = SortedListWithKey(range(10000), key=lambda val: val % 10)
    slt._check()

    values = sorted(range(10000), key=lambda val: (val % 10, val))
    assert slt == values
    assert all(val in slt for val in range(10000))

def test_add():
    random.seed(0)
    slt = SortedListWithKey(key=negate)
    for val in range(1000):
        slt.add(val)
        slt._check()

    slt = SortedListWithKey(key=negate)
    for val in range(1000, 0, -1):
        slt.add(val)
        slt._check()

    slt = SortedListWithKey(key=negate)
    for val in range(1000):
        slt.add(random.random())
        slt._check()

def test_update():
    slt = SortedListWithKey(key=negate)

    slt.update(range(1000))
    assert len(slt) == 1000
    slt._check()

    slt.update(range(100))
    assert len(slt) == 1100
    slt._check()

    slt.update(range(10000))
    assert len(slt) == 11100
    slt._check()

    values = sorted((val for val in chain(range(100), range(1000), range(10000))), key=negate)
    assert all(tup[0] == tup[1] for tup in zip(slt, values))

def test_contains():
    slt = SortedListWithKey(key=negate)
    assert 0 not in slt

    slt.update(range(10000))

    for val in range(10000):
        assert val in slt

    assert 10000 not in slt
    assert -1 not in slt

    slt._check()

def test_discard():
    slt = SortedListWithKey(key=negate)

    assert slt.discard(0) == None
    assert len(slt) == 0
    slt._check()

    slt = SortedListWithKey([1, 2, 2, 2, 3, 3, 5], load=4, key=negate)

    slt.discard(6)
    slt._check()
    slt.discard(4)
    slt._check()
    slt.discard(2)
    slt._check()

    assert all(tup[0] == tup[1] for tup in zip(slt, reversed([1, 2, 2, 3, 3, 5])))

def test_remove():
    slt = SortedListWithKey(key=negate)

    assert slt.discard(0) == None
    assert len(slt) == 0
    slt._check()

    slt = SortedListWithKey([1, 2, 2, 2, 3, 3, 5], load=4, key=negate)

    slt.remove(2)
    slt._check()

    assert all(tup[0] == tup[1] for tup in zip(slt, reversed([1, 2, 2, 3, 3, 5])))

@raises(ValueError)
def test_remove_valueerror1():
    slt = SortedListWithKey(key=negate)
    slt.remove(0)

@raises(ValueError)
def test_remove_valueerror2():
    slt = SortedListWithKey(range(100), load=10, key=negate)
    slt.remove(100)

@raises(ValueError)
def test_remove_valueerror3():
    slt = SortedListWithKey([1, 2, 2, 2, 3, 3, 5], key=negate)
    slt.remove(4)

def test_delete():
    slt = SortedListWithKey(range(20), load=4, key=negate)
    slt._check()
    for val in range(20):
        slt.remove(val)
        slt._check()
    assert len(slt) == 0
    assert slt._maxes == []
    assert slt._lists == []

def test_getitem():
    random.seed(0)
    slt = SortedListWithKey(load=17, key=negate)

    slt.append(5)
    assert slt[0] == 5
    slt.clear()

    lst = list()

    for rpt in range(100):
        val = random.random()
        slt.add(val)
        lst.append(val)

    lst.sort(reverse=True)

    assert all(slt[idx] == lst[idx] for idx in range(100))
    assert all(slt[idx - 99] == lst[idx - 99] for idx in range(100))

def test_getitem_slice():
    random.seed(0)
    slt = SortedListWithKey(load=17, key=negate)

    lst = list()

    for rpt in range(100):
        val = random.random()
        slt.add(val)
        lst.append(val)

    lst.sort(reverse=True)

    assert all(slt[start:] == lst[start:]
               for start in [-75, -25, 0, 25, 75])

    assert all(slt[:stop] == lst[:stop]
               for stop in [-75, -25, 0, 25, 75])

    assert all(slt[::step] == lst[::step]
               for step in [-5, -1, 1, 5])

    assert all(slt[start:stop] == lst[start:stop]
               for start in [-75, -25, 0, 25, 75]
               for stop in [-75, -25, 0, 25, 75])

    assert all(slt[:stop:step] == lst[:stop:step]
               for stop in [-75, -25, 0, 25, 75]
               for step in [-5, -1, 1, 5])

    assert all(slt[start::step] == lst[start::step]
               for start in [-75, -25, 0, 25, 75]
               for step in [-5, -1, 1, 5])

    assert all(slt[start:stop:step] == lst[start:stop:step]
               for start in [-75, -25, 0, 25, 75]
               for stop in [-75, -25, 0, 25, 75]
               for step in [-5, -1, 1, 5])

def test_getitem_slice_big():
    slt = SortedListWithKey(range(4), key=negate)
    lst = list(reversed(range(4)))

    itr = ((start, stop, step)
           for start in [-6, -4, -2, 0, 2, 4, 6]
           for stop in [-6, -4, -2, 0, 2, 4, 6]
           for step in [-3, -2, -1, 1, 2, 3])

    for start, stop, step in itr:
        assert slt[start:stop:step] == lst[start:stop:step]

@raises(ValueError)
def test_getitem_slicezero():
    slt = SortedListWithKey(range(100), load=17, key=negate)
    slt[::0]

@raises(IndexError)
def test_getitem_indexerror1():
    slt = SortedListWithKey(key=negate)
    slt[5]

@raises(IndexError)
def test_getitem_indexerror2():
    slt = SortedListWithKey(range(100), key=negate)
    slt[200]

@raises(IndexError)
def test_getitem_indexerror3():
    slt = SortedListWithKey(range(100), key=negate)
    slt[-101]

def test_delitem():
    random.seed(0)
    slt = SortedListWithKey(range(100), load=17, key=negate)
    while len(slt) > 0:
        del slt[random.randrange(len(slt))]
        slt._check()

def test_delitem_slice():
    slt = SortedListWithKey(range(100), load=17, key=negate)
    del slt[10:40:1]
    del slt[10:40:-1]
    del slt[10:40:2]
    del slt[10:40:-2]

def test_setitem():
    random.seed(0)
    slt = SortedListWithKey(range(0, 100, 10), load=4, key=negate)

    slt[-3] = 20
    slt._check()

    values = list(enumerate(range(95, 5, -10)))
    random.shuffle(values)
    for pos, val in values:
        slt[pos] = val

def test_setitem_slice():
    slt = SortedListWithKey(range(100), load=17, key=negate)
    slt[:10] = iter(range(99, 89, -1))
    assert slt == list(range(99, -1, -1))
    slt[:10:2] = iter([99, 97, 95, 93, 91])
    assert slt == list(range(99, -1, -1))
    slt[-50:] = range(49, -51, -1)
    assert slt == list(range(99, -51, -1))
    slt[-100:] = range(49, -1, -1)
    assert slt == list(range(99, -1, -1))
    slt[:] = range(99, -1, -1)
    assert slt == list(range(99, -1, -1))
    slt[90:] = []
    assert slt == list(range(99, 9, -1))
    slt[:10] = []
    assert slt == list(range(89, 9, -1))

@raises(ValueError)
def test_setitem_slice_bad():
    slt = SortedListWithKey(range(100), load=17, key=negate)
    slt[:10] = list(reversed(range(10)))

@raises(ValueError)
def test_setitem_slice_bad1():
    slt = SortedListWithKey(range(100), load=17, key=negate)
    slt[10:20] = range(20, 30)

@raises(ValueError)
def test_setitem_slice_bad2():
    slt = SortedListWithKey(range(100), load=17, key=negate)
    slt[20:30] = range(10, 20)

def test_setitem_extended_slice():
    slt = SortedListWithKey(range(1000, 0, -10), load=17, key=negate)
    lst = list(range(1000, 0, -10))
    lst[10:90:10] = range(905, 105, -100)
    slt[10:90:10] = range(905, 105, -100)
    assert slt == lst

@raises(ValueError)
def test_setitem_extended_slice_bad1():
    slt = SortedListWithKey(range(100), load=17, key=negate)
    slt[20:80:3] = list(range(10))

@raises(ValueError)
def test_setitem_extended_slice_bad2():
    slt = SortedListWithKey(range(100), load=17, key=negate)
    slt[40:90:5] = list(range(10))

@raises(ValueError)
def test_setitem_valueerror1():
    slt = SortedListWithKey(range(10), key=negate)
    slt[9] = 10

@raises(ValueError)
def test_setitem_valueerror2():
    slt = SortedListWithKey(range(10), key=negate)
    slt[0] = 0

def test_iter():
    slt = SortedListWithKey(range(10000), key=negate)
    itr = iter(slt)
    assert all(tup[0] == tup[1] for tup in zip(range(9999, -1, -1), itr))

def test_reversed():
    slt = SortedListWithKey(range(10000), key=negate)
    rev = reversed(slt)
    assert all(tup[0] == tup[1] for tup in zip(range(10000), rev))

def test_islice():
    return
    slt = SortedListWithKey(load=7, key=negate)

    assert [] == list(slt.islice())

    values = sorted(range(53), key=negate)
    slt.update(values)

    for start in range(53):
        for stop in range(53):
            assert list(slt.islice(start, stop)) == values[start:stop]

    for start in range(53):
        for stop in range(53):
            assert list(slt.islice(start, stop, reverse=True)) == values[start:stop][::-1]

    for start in range(53):
        assert list(slt.islice(start=start)) == values[start:]
        assert list(slt.islice(start=start, reverse=True)) == values[start:][::-1]

    for stop in range(53):
        assert list(slt.islice(stop=stop)) == values[:stop]
        assert list(slt.islice(stop=stop, reverse=True)) == values[:stop][::-1]

def test_irange():
    slt = SortedListWithKey(load=7, key=negate)

    assert [] == list(slt.irange())

    values = list(range(53))
    slt.update(values)

    for start in range(53):
        for end in range(start, 53):
            assert list(slt.irange(end, start)) == values[start:(end + 1)][::-1]
            assert list(slt.irange(end, start, reverse=True)) == values[start:(end + 1)]

    for start in range(53):
        for end in range(start, 53):
            assert list(slt.irange(end, start, (True, False))) == values[(start + 1):(end + 1)][::-1]

    for start in range(53):
        for end in range(start, 53):
            assert list(slt.irange(end, start, (False, True))) == values[start:end][::-1]

    for start in range(53):
        for end in range(start, 53):
            assert list(slt.irange(end, start, (False, False))) == values[(start + 1):end][::-1]

    for start in range(53):
        assert list(slt.irange(start)) == values[:(start + 1)][::-1]

    for end in range(53):
        assert list(slt.irange(None, end, (True, False))) == values[(end + 1):][::-1]

    assert list(slt.irange(inclusive=(False, False))) == values[::-1]

    assert list(slt.irange(-1)) == []
    assert list(slt.irange(None, -1, (True, False))) == values[::-1]

def test_len():
    slt = SortedListWithKey(key=negate)

    for val in range(10000):
        slt.add(val)
        assert len(slt) == (val + 1)

def test_bisect_left():
    slt = SortedListWithKey(key=negate)
    assert slt.bisect_left(0) == 0
    slt = SortedListWithKey(range(100), load=17, key=negate)
    slt.update(range(100))
    slt._check()
    assert slt.bisect_left(50) == 98
    assert slt.bisect_left(0) == 198
    assert slt.bisect_left(-1) == 200

def test_bisect():
    slt = SortedListWithKey(key=negate)
    assert slt.bisect(10) == 0
    slt = SortedListWithKey(range(100), load=17, key=negate)
    slt.update(range(100))
    slt._check()
    assert slt.bisect(10) == 180
    assert slt.bisect(0) == 200

def test_bisect_right():
    slt = SortedListWithKey(key=negate)
    assert slt.bisect_right(10) == 0
    slt = SortedListWithKey(range(100), load=17, key=negate)
    slt.update(range(100))
    slt._check()
    assert slt.bisect_right(10) == 180
    assert slt.bisect_right(0) == 200

def test_copy():
    slt = SortedListWithKey(range(100), load=7, key=negate)
    two = slt.copy()
    slt.add(100)
    assert len(slt) == 101
    assert len(two) == 100

def test_copy_copy():
    import copy
    slt = SortedListWithKey(range(100), load=7, key=negate)
    two = copy.copy(slt)
    slt.add(100)
    assert len(slt) == 101
    assert len(two) == 100

def test_count():
    slt = SortedListWithKey(load=7, key=negate)

    assert slt.count(0) == 0

    for iii in range(100):
        for jjj in range(iii):
            slt.add(iii)
        slt._check()

    for iii in range(100):
        assert slt.count(iii) == iii

def test_append():
    slt = SortedListWithKey(load=17, key=negate)

    slt.append(1000)

    for val in range(999, -1, -1):
        slt.append(val)
        slt._check()

@raises(ValueError)
def test_append_valueerror():
    slt = SortedListWithKey(range(100), key=negate)
    slt.append(5)

def test_extend():
    slt = SortedListWithKey(load=17, key=negate)

    slt.extend(range(300, 200, -1))
    slt._check()

    slt.extend(list(range(200, 100, -1)))
    slt._check()

    for val in range(100, 0, -1):
        del slt._index[:]
        slt._build_index()
        slt.extend([val] * (101 - val))
        slt._check()

@raises(ValueError)
def test_extend_valueerror1():
    slt = SortedListWithKey(key=negate)
    slt.extend([1, 2, 3, 5, 4, 6])

@raises(ValueError)
def test_extend_valueerror2():
    slt = SortedListWithKey(range(20), load=4, key=negate)
    slt.extend([5, 4, 3, 2, 1])

def test_insert():
    slt = SortedListWithKey(range(10), load=4, key=negate)
    slt.insert(-1, 0)
    slt._check()
    slt.insert(-100, 9)
    slt._check()
    slt.insert(0, 10)
    slt._check()
    slt.insert(14, -1)
    slt._check()

    slt = SortedListWithKey(load=4, key=negate)
    slt.insert(0, 5)
    slt._check()

    slt = SortedListWithKey(range(5, 15), load=4, key=negate)
    for rpt in range(8):
        slt.insert(0, 15)
        slt._check()

    slt = SortedListWithKey(range(10), load=4, key=negate)
    slt.insert(8, 2)
    slt._check()

@raises(ValueError)
def test_insert_valueerror1():
    slt = SortedListWithKey(range(10), load=4, key=negate)
    slt.insert(10, 5)

@raises(ValueError)
def test_insert_valueerror2():
    slt = SortedListWithKey(range(10), load=4, key=negate)
    slt.insert(0, 0)

@raises(ValueError)
def test_insert_valueerror3():
    slt = SortedListWithKey(range(10), load=4, key=negate)
    slt.insert(5, 3)

@raises(ValueError)
def test_insert_valueerror4():
    slt = SortedListWithKey(range(10), load=4, key=negate)
    slt.insert(5, 7)

def test_pop():
    slt = SortedListWithKey(range(10), load=4, key=negate)
    slt._check()
    assert slt.pop() == 0
    slt._check()
    assert slt.pop(0) == 9
    slt._check()
    assert slt.pop(-2) == 2
    slt._check()
    assert slt.pop(4) == 4
    slt._check()

@raises(IndexError)
def test_pop_indexerror1():
    slt = SortedListWithKey(range(10), load=4, key=negate)
    slt.pop(-11)

@raises(IndexError)
def test_pop_indexerror2():
    slt = SortedListWithKey(range(10), load=4, key=negate)
    slt.pop(10)

def test_index():
    slt = SortedListWithKey(range(100), load=17, key=negate)

    for pos, val in enumerate(range(99, -1, -1)):
        assert val == slt.index(pos)

    assert slt.index(99, 0, 1000) == 0

    slt = SortedListWithKey((0 for rpt in range(100)), load=17, key=negate)

    for start in range(100):
        for stop in range(start, 100):
            assert slt.index(0, start, stop + 1) == start

    for start in range(100):
        assert slt.index(0, -(100 - start)) == start

    assert slt.index(0, -1000) == 0

@raises(ValueError)
def test_index_valueerror1():
    slt = SortedListWithKey([0] * 10, load=4, key=negate)
    slt.index(0, 10)

@raises(ValueError)
def test_index_valueerror2():
    slt = SortedListWithKey([0] * 10, load=4, key=negate)
    slt.index(0, 0, -10)

@raises(ValueError)
def test_index_valueerror3():
    slt = SortedListWithKey([0] * 10, load=4, key=negate)
    slt.index(0, 7, 3)

@raises(ValueError)
def test_index_valueerror4():
    slt = SortedListWithKey([0] * 10, load=4, key=negate)
    slt.index(1)

@raises(ValueError)
def test_index_valueerror5():
    slt = SortedListWithKey(key=negate)
    slt.index(1)

@raises(ValueError)
def test_index_valueerror6():
    slt = SortedListWithKey(range(10), load=4, key=negate)
    slt.index(6, 5)

def test_mul():
    this = SortedListWithKey(range(10), load=4, key=negate)
    that = this * 5
    this._check()
    that._check()
    assert this == list(reversed((range(10))))
    assert that == list(sorted(list(range(10)) * 5, reverse=True))
    assert this != that

def test_imul():
    this = SortedListWithKey(range(10), load=4, key=negate)
    this *= 5
    this._check()
    assert this == sorted(list(range(10)) * 5, reverse=True)

def test_op_add():
    this = SortedListWithKey(range(10), load=4, key=negate)
    assert (this + this + this) == (this * 3)

    that = SortedListWithKey(range(10), load=4, key=negate)
    that += that
    that += that
    assert that == (this * 4)

def test_eq():
    this = SortedListWithKey(range(10), load=4, key=negate)
    that = SortedListWithKey(range(20), load=4, key=negate)
    assert not (this == that)
    that.clear()
    that.update(range(10))
    assert this == that

def test_lt():
    this = SortedListWithKey(range(10), load=4, key=negate)
    that = SortedListWithKey(range(10, 20), load=5, key=negate)
    assert this < that
    assert not (that < this)
    that = SortedListWithKey(range(1, 20), load=6, key=negate)
    assert this < that
    that = SortedListWithKey(range(1, 10), load=4, key=negate)
    assert not (this < that)

def test_lte():
    this = SortedListWithKey(range(10), load=4, key=negate)
    that = SortedListWithKey(range(10), load=5, key=negate)
    assert this <= that
    assert that <= this
    del this[-1]
    assert this <= that
    assert not (that <= this)

def test_gt():
    this = SortedListWithKey(range(10), load=4, key=negate)
    that = SortedListWithKey(range(10, 20), load=5, key=negate)
    assert that > this
    assert not (this > that)
    that = SortedListWithKey(range(1, 20), load=6, key=negate)
    assert that > this
    that = SortedListWithKey(range(1, 10), load=4, key=negate)
    assert not (that > this)

def test_gte():
    this = SortedListWithKey(range(10), load=4, key=negate)
    that = SortedListWithKey(range(10), load=5, key=negate)
    assert this >= that
    assert that >= this
    del this[-1]
    assert that >= this
    assert not (this >= that)

def test_repr():
    this = SortedListWithKey(range(10), load=4, key=negate)
    assert repr(this).startswith('SortedListWithKey([9, 8, 7, 6, 5, 4, 3, 2, 1, 0], key=<function negate at ')

def test_pickle():
    import pickle
    alpha = SortedListWithKey(range(10000), key=negate, load=500)
    beta = pickle.loads(pickle.dumps(alpha))
    assert alpha == beta
    assert alpha._key == beta._key
    assert alpha._load == beta._load

@raises(AssertionError)
def test_check():
    slt = SortedListWithKey(range(10), load=4, key=negate)
    slt._len = 5
    slt._check()

if __name__ == '__main__':
    import nose
    nose.main()



"""
Benchmark Sorted Set Datatypes
"""

import warnings
from sys import hexversion

from .benchmark import *

if hexversion < 0x03000000:
    range = xrange

# Tests.

@register_test
def contains(func, size):
    for val in lists[size][::100]:
        assert func(val)

@register_test
def iter(func, size):
    assert all(idx == val for idx, val in enumerate(func()))

@register_test
def add(func, size):
    for val in lists[size][::100]:
        func(-val)

@register_test
def update_tiny(func, size):
    func(range(size, size + 10))

@register_test
def update_small(func, size):
    func(range(size, size + int(size / 10)))

@register_test
def update_medium(func, size):
    func(range(size, size + int(size * 5 / 10)))

@register_test
def update_large(func, size):
    func(range(size, size + int(size * 9 / 10)))

@register_test
def union_tiny(func, size):
    func(range(size, size + 10))

@register_test
def union_small(func, size):
    func(range(size, size + int(size / 10)))

@register_test
def union_medium(func, size):
    func(range(size, size + int(size * 5 / 10)))

@register_test
def union_large(func, size):
    func(range(size, size + int(size * 9 / 10)))

@register_test
def remove(func, size):
    for val in lists[size][::100]:
        func(val)

@register_test
def difference_tiny(func, size):
    func(range(int(size / 2), int(size / 2) + 10))

@register_test
def difference_small(func, size):
    func(range(int(size / 2), int(size / 2) + int(size / 10)))

@register_test
def difference_medium(func, size):
    func(range(int(size / 2), int(size / 2) + int(size * 5 / 10)))

@register_test
def difference_large(func, size):
    func(range(int(size / 2), int(size / 2) + int(size * 9 / 10)))

@register_test
def difference_update_tiny(func, size):
    func(range(int(size / 2), int(size / 2) + 10))

@register_test
def difference_update_small(func, size):
    func(range(int(size / 2), int(size / 2) + int(size / 10)))

@register_test
def difference_update_medium(func, size):
    func(range(int(size / 2), int(size / 2) + int(size * 5 / 10)))

@register_test
def difference_update_large(func, size):
    func(range(int(size / 2), int(size / 2) + int(size * 9 / 10)))

@register_test
def intersection_tiny(func, size):
    func(range(int(size / 2), int(size / 2) + 10))

@register_test
def intersection_small(func, size):
    func(range(int(size / 2), int(size / 2) + int(size / 10)))

@register_test
def intersection_medium(func, size):
    func(range(int(size / 2), int(size / 2) + int(size * 5 / 10)))

@register_test
def intersection_large(func, size):
    func(range(int(size / 2), int(size / 2) + int(size * 9 / 10)))

@register_test
def intersection_update_tiny(func, size):
    func(range(int(size / 2), int(size / 2) + 10))

@register_test
def intersection_update_small(func, size):
    func(range(int(size / 2), int(size / 2) + int(size / 10)))

@register_test
def intersection_update_medium(func, size):
    func(range(int(size / 2), int(size / 2) + int(size * 5 / 10)))

@register_test
def intersection_update_large(func, size):
    func(range(int(size / 2), int(size / 2) + int(size * 9 / 10)))

@register_test
def symmetric_difference_tiny(func, size):
    func(range(int(size / 2), int(size / 2) + 10))

@register_test
def symmetric_difference_small(func, size):
    func(range(int(size / 2), int(size / 2) + int(size / 10)))

@register_test
def symmetric_difference_medium(func, size):
    func(range(int(size / 2), int(size / 2) + int(size * 5 / 10)))

@register_test
def symmetric_difference_large(func, size):
    func(range(int(size / 2), int(size / 2) + int(size * 9 / 10)))

@register_test
def symmetric_difference_update_tiny(func, size):
    func(range(int(size / 2), int(size / 2) + 10))

@register_test
def symmetric_difference_update_small(func, size):
    func(range(int(size / 2), int(size / 2) + int(size / 10)))

@register_test
def symmetric_difference_update_medium(func, size):
    func(range(int(size / 2), int(size / 2) + int(size * 5 / 10)))

@register_test
def symmetric_difference_update_large(func, size):
    func(range(int(size / 2), int(size / 2) + int(size * 9 / 10)))

@register_test
def pop(func, size):
    for rpt in range(int(size / 100)):
        func()

@register_test
def init(func, size):
    func(lists[size])

# Setups.

def do_nothing(obj, size):
    pass

def fill_values(obj, size):
    if hasattr(obj, 'update'):
        obj.update(sorted(lists[size]))
    else:
        for val in lists[size]:
            obj.add(val)

# Implementation imports.

from .context import sortedcontainers
from sortedcontainers import SortedSet
kinds['SortedSet'] = SortedSet

try:
    from rbtree import rbset
    kinds['rbset'] = rbset
except ImportError:
    warnings.warn('No module named rbtree', ImportWarning)

try:
    from blist import sortedset
    kinds['blist.sortedset'] = sortedset
except ImportError:
    warnings.warn('No module named blist', ImportWarning)

try:
    from skiplistcollections import SkipListSet
    kinds['SkipListSet'] = SkipListSet
except ImportError:
    warnings.warn('No module named skiplistcollections', ImportWarning)

try:
    from banyan import SortedSet as BanyanSortedSet
    kinds['banyan.SortedSet'] = BanyanSortedSet
except ImportError:
    warnings.warn('No module named banyan', ImportWarning)

# Implementation configuration.

from .benchmark import remove

for name in tests:
    impls[name] = OrderedDict()

for name, kind in kinds.items():
    impls['contains'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': '__contains__',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['iter'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': '__iter__',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['add'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'add',
        'limit': 1000000
    }

limit('add', 'blist.sortedset', 10000)

for name, kind in kinds.items():
    impls['update_tiny'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'update',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['update_small'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'update',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['update_medium'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'update',
        'limit': 1000000
    }

limit('update_medium', 'blist.sortedset', 10000)

for name, kind in kinds.items():
    impls['update_large'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'update',
        'limit': 1000000
    }

limit('update_large', 'blist.sortedset', 10000)

remove('update_tiny', 'SkipListSet')
remove('update_small', 'SkipListSet')
remove('update_medium', 'SkipListSet')
remove('update_large', 'SkipListSet')

for name, kind in kinds.items():
    impls['union_tiny'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'union',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['union_small'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'union',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['union_medium'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'union',
        'limit': 1000000
    }

limit('union_medium', 'blist.sortedset', 10000)

for name, kind in kinds.items():
    impls['union_large'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'union',
        'limit': 1000000
    }

limit('union_large', 'blist.sortedset', 10000)

remove('union_tiny', 'SkipListSet')
remove('union_small', 'SkipListSet')
remove('union_medium', 'SkipListSet')
remove('union_large', 'SkipListSet')

for name, kind in kinds.items():
    impls['remove'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'remove',
        'limit': 1000000
    }

limit('remove', 'blist.sortedset', 10000)

for name, kind in kinds.items():
    impls['difference_tiny'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'difference',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['difference_small'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'difference',
        'limit': 1000000
    }

limit('difference_small', 'rbset', 10000)

for name, kind in kinds.items():
    impls['difference_medium'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'difference',
        'limit': 1000000
    }

limit('difference_medium', 'rbset', 10000)
limit('difference_medium', 'blist.sortedset', 10000)

for name, kind in kinds.items():
    impls['difference_large'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'difference',
        'limit': 1000000
    }

limit('difference_large', 'rbset', 10000)
limit('difference_large', 'blist.sortedset', 10000)

remove('difference_tiny', 'SkipListSet')
remove('difference_small', 'SkipListSet')
remove('difference_medium', 'SkipListSet')
remove('difference_large', 'SkipListSet')

for name, kind in kinds.items():
    impls['difference_update_tiny'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'difference_update',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['difference_update_small'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'difference_update',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['difference_update_medium'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'difference_update',
        'limit': 1000000
    }

limit('difference_update_medium', 'blist.sortedset', 10000)

for name, kind in kinds.items():
    impls['difference_update_large'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'difference_update',
        'limit': 1000000
    }

limit('difference_update_large', 'blist.sortedset', 10000)

remove('difference_update_tiny', 'SkipListSet')
remove('difference_update_small', 'SkipListSet')
remove('difference_update_medium', 'SkipListSet')
remove('difference_update_large', 'SkipListSet')

for name, kind in kinds.items():
    impls['intersection_tiny'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'intersection',
        'limit': 1000000
    }

limit('intersection_tiny', 'blist.sortedset', 10000)

for name, kind in kinds.items():
    impls['intersection_small'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'intersection',
        'limit': 1000000
    }

limit('intersection_small', 'rbset', 10000)
limit('intersection_small', 'blist.sortedset', 10000)

for name, kind in kinds.items():
    impls['intersection_medium'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'intersection',
        'limit': 1000000
    }

limit('intersection_medium', 'rbset', 10000)
limit('intersection_medium', 'blist.sortedset', 10000)

for name, kind in kinds.items():
    impls['intersection_large'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'intersection',
        'limit': 1000000
    }

limit('intersection_large', 'rbset', 10000)
limit('intersection_large', 'blist.sortedset', 10000)

remove('intersection_tiny', 'SkipListSet')
remove('intersection_small', 'SkipListSet')
remove('intersection_medium', 'SkipListSet')
remove('intersection_large', 'SkipListSet')

for name, kind in kinds.items():
    impls['intersection_update_tiny'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'intersection_update',
        'limit': 1000000
    }

limit('intersection_update_tiny', 'rbset', 10000)
limit('intersection_update_tiny', 'blist.sortedset', 10000)

for name, kind in kinds.items():
    impls['intersection_update_small'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'intersection_update',
        'limit': 1000000
    }

limit('intersection_update_small', 'rbset', 10000)
limit('intersection_update_small', 'blist.sortedset', 10000)

for name, kind in kinds.items():
    impls['intersection_update_medium'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'intersection_update',
        'limit': 1000000
    }

limit('intersection_update_medium', 'rbset', 10000)
limit('intersection_update_medium', 'blist.sortedset', 10000)

for name, kind in kinds.items():
    impls['intersection_update_large'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'intersection_update',
        'limit': 1000000
    }

limit('intersection_update_large', 'rbset', 10000)
limit('intersection_update_large', 'blist.sortedset', 10000)

remove('intersection_update_tiny', 'SkipListSet')
remove('intersection_update_small', 'SkipListSet')
remove('intersection_update_medium', 'SkipListSet')
remove('intersection_update_large', 'SkipListSet')

for name, kind in kinds.items():
    impls['symmetric_difference_tiny'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'symmetric_difference',
        'limit': 1000000
    }

limit('symmetric_difference_tiny', 'blist.sortedset', 10000)

for name, kind in kinds.items():
    impls['symmetric_difference_small'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'symmetric_difference',
        'limit': 1000000
    }

limit('symmetric_difference_small', 'rbset', 10000)
limit('symmetric_difference_small', 'blist.sortedset', 10000)

for name, kind in kinds.items():
    impls['symmetric_difference_medium'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'symmetric_difference',
        'limit': 1000000
    }

limit('symmetric_difference_medium', 'rbset', 10000)
limit('symmetric_difference_medium', 'blist.sortedset', 10000)

for name, kind in kinds.items():
    impls['symmetric_difference_large'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'symmetric_difference',
        'limit': 1000000
    }

limit('symmetric_difference_large', 'rbset', 10000)
limit('symmetric_difference_large', 'blist.sortedset', 10000)

remove('symmetric_difference_tiny', 'SkipListSet')
remove('symmetric_difference_small', 'SkipListSet')
remove('symmetric_difference_medium', 'SkipListSet')
remove('symmetric_difference_large', 'SkipListSet')

for name, kind in kinds.items():
    impls['symmetric_difference_update_tiny'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'symmetric_difference_update',
        'limit': 1000000
    }

limit('symmetric_difference_update_tiny', 'blist.sortedset', 10000)

for name, kind in kinds.items():
    impls['symmetric_difference_update_small'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'symmetric_difference_update',
        'limit': 1000000
    }

limit('symmetric_difference_update_small', 'rbset', 10000)
limit('symmetric_difference_update_small', 'blist.sortedset', 10000)

for name, kind in kinds.items():
    impls['symmetric_difference_update_medium'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'symmetric_difference_update',
        'limit': 1000000
    }

limit('symmetric_difference_update_medium', 'rbset', 10000)
limit('symmetric_difference_update_medium', 'blist.sortedset', 10000)

for name, kind in kinds.items():
    impls['symmetric_difference_update_large'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'symmetric_difference_update',
        'limit': 1000000
    }

limit('symmetric_difference_update_large', 'rbset', 10000)
limit('symmetric_difference_update_large', 'blist.sortedset', 10000)

remove('symmetric_difference_update_tiny', 'rbset')
remove('symmetric_difference_update_small', 'rbset')
remove('symmetric_difference_update_medium', 'rbset')
remove('symmetric_difference_update_large', 'rbset')
remove('symmetric_difference_update_tiny', 'SkipListSet')
remove('symmetric_difference_update_small', 'SkipListSet')
remove('symmetric_difference_update_medium', 'SkipListSet')
remove('symmetric_difference_update_large', 'SkipListSet')

for name, kind in kinds.items():
    impls['pop'][name] = {
        'setup': fill_values,
        'ctor': kind,
        'func': 'pop',
        'limit': 1000000
    }

for name, kind in kinds.items():
    impls['init'][name] = {
        'setup': do_nothing,
        'ctor': kind,
        'func': 'update',
        'limit': 1000000
    }

remove('init', 'SkipListSet')

limit('init', 'blist.sortedset', 100000)

if __name__ == '__main__':
    main('SortedSet')
